import { test } from '@japa/runner'
import { Env } from '@nestmtx/config'
import Joi from 'joi'
import { inspect } from 'util'
import type { Paths } from '../src/ClientTypes'
import { DocFoxClient } from '../src/DocFoxClient'

test.group('DocFox Operation Methods', (group) => {
  const env = new Env({
    KEY: Joi.string().required(),
    SECRET: Joi.string().required(),
    UPDATE_KYC_APPLICATION_RISK_RATINGS_QUESTIONS_PARAMS: Joi.object().required(),
    UPDATE_KYC_APPLICATION_RISK_RATINGS_QUESTIONS_PAYLOAD: Joi.object().required(),
  })
  const client = new DocFoxClient(env.get('KEY'), env.get('SECRET'))
  group.tap((test) => test.tags(['@operations']))

  test('updateKycApplicationRiskRating', async ({ assert }) => {
    const params = env.get('UPDATE_KYC_APPLICATION_RISK_RATINGS_QUESTIONS_PARAMS')
    const payload = env.get('UPDATE_KYC_APPLICATION_RISK_RATINGS_QUESTIONS_PAYLOAD')
    const response = await client.updateKycApplicationRiskRating(params, payload)
    assert.exists(response)
    assert.exists(response.status)
    assert.equal(
      response.status,
      400,
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
    assert.exists(
      response.data,
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
    assert.exists(
      (response.data as Paths.UpdateKycApplicationRiskRating.Responses.$400).errors,
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
    assert.isArray(
      (response.data as Paths.UpdateKycApplicationRiskRating.Responses.$400).errors,
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
    assert.equal(
      (response.data as Paths.UpdateKycApplicationRiskRating.Responses.$400).errors.length,
      1,
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
    assert.equal(
      (response.data as Paths.UpdateKycApplicationRiskRating.Responses.$400).errors[0].detail,
      'Risk rating already completed',
      inspect(
        {
          params,
          payload,
          status: response.status,
          response: response.data,
        },
        false,
        10,
        true
      )
    )
  }).tags(['@updateKycApplicationRiskRating'])
})
