import { test } from '@japa/runner'
import axios from 'axios'
import crypto from 'crypto'
import { EventEmitter } from 'events'
import { DateTime } from 'luxon'
import { inspect } from 'util'
import DocFoxClient from '../'
import { OperationMethodResponses } from '../src/DocFoxClient'
import { DocFoxAuthenticationError } from '../src/Errors'
import { isClass } from './utils'

test.group('DocFoxClient Typescript Package', () => {
  test('Environment is Setup', ({ assert }) => {
    assert.isString(process.env.KEY)
    assert.isString(process.env.SECRET)
  })
  test('DocFoxClient is a Class', ({ assert }) => {
    assert.isDefined(DocFoxClient)
    assert.isTrue(isClass(DocFoxClient), 'DocFoxClient should be a class')
  })
  test('Can create an instance of DocFoxClient', ({ assert }) => {
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!)
    assert.isDefined(client)
    assert.instanceOf(client, DocFoxClient)
    assert.isFunction(client.getUri)
    assert.isFunction(client.request)
    assert.isFunction(client.get)
    assert.isFunction(client.delete)
    assert.isFunction(client.head)
    assert.isFunction(client.options)
    assert.isFunction(client.post)
    assert.isFunction(client.put)
    assert.isFunction(client.patch)
    assert.isFunction(client.postForm)
    assert.isFunction(client.putForm)
    assert.isFunction(client.patchForm)
    assert.isFunction(client.requestAuthentication)
    assert.isFunction(client.requestToken)
    assert.isFunction(client.listKycApplication)
    assert.isFunction(client.createKycApplication)
    assert.isFunction(client.getKycApplication)
    assert.isFunction(client.deleteKycApplication)
    assert.isFunction(client.updateKycApplicationApprove)
    assert.isFunction(client.updateKycApplicationUnapprove)
    assert.isFunction(client.updateKycApplicationTransfer)
    assert.isFunction(client.archiveKycApplication)
    assert.isFunction(client.unarchiveKycApplication)
    assert.isFunction(client.listKycApplicationStatusSummarie)
    assert.isFunction(client.listKycApplicationProfile)
    assert.isFunction(client.getProfile)
    assert.isFunction(client.listProfileName)
    assert.isFunction(client.createProfileName)
    assert.isFunction(client.getName)
    assert.isFunction(client.deleteName)
    assert.isFunction(client.updateName)
    assert.isFunction(client.listProfileNumber)
    assert.isFunction(client.createProfileNumber)
    assert.isFunction(client.getNumber)
    assert.isFunction(client.deleteNumber)
    assert.isFunction(client.updateNumber)
    assert.isFunction(client.listProfileContact)
    assert.isFunction(client.createProfileContact)
    assert.isFunction(client.getContact)
    assert.isFunction(client.deleteContact)
    assert.isFunction(client.updateContact)
    assert.isFunction(client.listProfileAddresse)
    assert.isFunction(client.createProfileAddresse)
    assert.isFunction(client.getAddresse)
    assert.isFunction(client.deleteAddresse)
    assert.isFunction(client.updateAddresse)
    assert.isFunction(client.listProfileAdditionalDetail)
    assert.isFunction(client.createProfileAdditionalDetail)
    assert.isFunction(client.getAdditionalDetail)
    assert.isFunction(client.deleteAdditionalDetail)
    assert.isFunction(client.updateAdditionalDetail)
    assert.isFunction(client.listContactInvitationToken)
    assert.isFunction(client.createContactInvitationToken)
    assert.isFunction(client.getInvitationToken)
    assert.isFunction(client.updateInvitationToken)
    assert.isFunction(client.listContactDataRequirementPermission)
    assert.isFunction(client.getDataRequirementPermission)
    assert.isFunction(client.deleteDataRequirementPermission)
    assert.isFunction(client.createDataRequirementPermission)
    assert.isFunction(client.listDataRequirementDataRequirementPermission)
    assert.isFunction(client.createDataRequirementDataRequirementPermission)
    assert.isFunction(client.createContactDataRequirementPermission)
    assert.isFunction(client.listKycApplicationDataRequirement)
    assert.isFunction(client.getDataRequirement)
    assert.isFunction(client.listKycEntityTemplate)
    assert.isFunction(client.getKycEntityTemplate)
    assert.isFunction(client.listKycApplicationDataComparison)
    assert.isFunction(client.getDataComparison)
    assert.isFunction(client.listProfileDataComparison)
    assert.isFunction(client.listDataRequirementDataComparison)
    assert.isFunction(client.listDataComparisonComparisonResult)
    assert.isFunction(client.getComparisonResult)
    assert.isFunction(client.listDataRequirementEvidenceSubmission)
    assert.isFunction(client.getEvidenceSubmission)
    assert.isFunction(client.updateEvidenceSubmission)
    assert.isFunction(client.updateEvidenceSubmissionReject)
    assert.isFunction(client.updateEvidenceSubmissionApprove)
    assert.isFunction(client.updateEvidenceSubmissionReplace)
    assert.isFunction(client.listEvidenceSubmissionQualitativeCheck)
    assert.isFunction(client.getQualitativeCheck)
    assert.isFunction(client.listEvidenceSubmissionRejectionReason)
    assert.isFunction(client.getRejectionReason)
    assert.isFunction(client.listEvidenceSubmissionDocument)
    assert.isFunction(client.createEvidenceSubmissionDocument)
    assert.isFunction(client.getDocument)
    assert.isFunction(client.listDocumentDocumentToken)
    assert.isFunction(client.createDocumentDocumentToken)
    assert.isFunction(client.getDocumentToken)
    assert.isFunction(client.getDocumentFileDownload)
    assert.isFunction(client.getForm)
    assert.isFunction(client.getFormFormData)
    assert.isFunction(client.createForm)
    assert.isFunction(client.getFormDataById)
    assert.isFunction(client.listKycApplicationRelatedEntitie)
    assert.isFunction(client.createKycApplicationRelatedEntitie)
    assert.isFunction(client.getRelatedEntitie)
    assert.isFunction(client.deleteRelatedEntitie)
    assert.isFunction(client.listKycApplicationPdfReportSummarie)
    assert.isFunction(client.createKycApplicationPdfReportSummarie)
    assert.isFunction(client.getPdfReportSummarie)
    assert.isFunction(client.getPdfReportSummarieDownload)
    assert.isFunction(client.listUser)
    assert.isFunction(client.getUser)
    assert.isFunction(client.getRiskRatingEvaluation)
    assert.isFunction(client.listKycApplicationRiskRatingEvaluation)
    assert.isFunction(client.listKycApplicationRiskRating)
    assert.isFunction(client.createKycApplicationRiskRating)
    assert.isFunction(client.getKycApplicationRiskRating)
    assert.isFunction(client.listKycApplicationRiskRatingsQuestion)
    assert.isFunction(client.updateKycApplicationRiskRatingsQuestions)
    assert.isFunction(client.getBankAccountSchema)
    assert.isFunction(client.listKycApplicationBankAccount)
    assert.isFunction(client.createKycApplicationBankAccount)
    assert.isFunction(client.getBankAccount)
    assert.isFunction(client.getBankAccountVerification)
    assert.isFunction(client.updateBankAccountVerification)
    assert.isFunction(client.listBankAccountBankAccountVerification)
    assert.isFunction(client.listKycApplicationConsumerReportQuerie)
    assert.isFunction(client.getConsumerReportQuerie)
    assert.isFunction(client.getConsumerReportQuerieDownload)
    assert.isFunction(client.createOnboardingInvitation)
    assert.isFunction(client.updateOnboardingInvitation)
    assert.isFunction(client.listFormFormPdfExport)
    assert.isFunction(client.createFormFormPdfExport)
    assert.isFunction(client.getFormPdfExport)
    assert.isFunction(client.getFormPdfExportDownload)
    assert.isFunction(client.listKycEntityTemplateDesignationTemplate)
    assert.isFunction(client.getRelatedEntityDesignation)
    assert.isFunction(client.deleteRelatedEntityDesignation)
    assert.isFunction(client.listRelatedEntitieRelatedEntityDesignation)
    assert.isFunction(client.createRelatedEntitieRelatedEntityDesignation)
    assert.isFunction(client.listProductProductProductAddon)
    assert.isFunction(client.listAccountApplicationAccountAddon)
    assert.isFunction(client.getAccountAddon)
    assert.isFunction(client.listProductDisclosure)
    assert.isFunction(client.listDisclosure)
    assert.isFunction(client.getDisclosure)
    assert.isFunction(client.listProduct)
    assert.isFunction(client.getProduct)
    assert.isFunction(client.listKycEntityTemplateProduct)
    assert.isFunction(client.getAccountApplicationAccountSummaryPdf)
    assert.isFunction(client.listKycEntityTemplateDisclosure)
    assert.isFunction(client.listAccountApplicationDataRequirement)
    assert.isFunction(client.listProductProductAddonsDisclosure)
    assert.isFunction(client.getAccountApplicationAccountSummary)
    assert.isFunction(client.listKycApplicationAccountApplication)
    assert.isFunction(client.createKycApplicationAccountApplication)
    assert.isFunction(client.getAccountApplication)
    assert.isFunction(client.updateAccountApplication)
    assert.isFunction(client.updateAccountApplicationDisclosureAcceptances)
  })
  test('Definitions are Defined', ({ assert }) => {
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!)
    assert.isDefined(client.definitions)
    assert.isObject(client.definitions)
  })
  test('Can make a request to fetch a list of users using an operation method', async ({
    assert,
  }) => {
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!)
    const {
      status,
      data,
      config: { metadata },
    } = await client.listUser({
      per_page: 1,
    })
    assert.equal(status, 200, inspect(data, false, 10, true))
    assert.isObject(data)
    assert.isArray(data.data)
    assert.isObject(data.meta)
    assert.isTrue(metadata.reauthenticated)
  }).timeout(10000)
  test('Can make multiple requests without requiring re-authentication', async ({ assert }) => {
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!)
    let test = 0
    while (test < 5) {
      const {
        status,
        data,
        config: { metadata },
      } = await client.listUser({
        per_page: 1,
      })
      assert.equal(status, 200, inspect(data, false, 10, true))
      assert.isObject(data)
      assert.isArray(data.data)
      assert.isObject(data.meta)
      if (test > 0) {
        assert.isUndefined(metadata.reauthenticated)
      } else {
        assert.isTrue(metadata.reauthenticated)
      }
      test++
    }
  }).timeout(10000)
  test('Can use a custom authentication generation function', async ({ assert }) => {
    const bus = new EventEmitter()
    const usedCustomAuthenticator = new Promise((resolve, reject) => {
      bus.once('custom-authenticator', resolve)
      bus.once('finished', reject)
      bus.once('error', reject)
    })
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!, async (key, secret) => {
      const authclient = axios.create({
        baseURL: 'https://www.docfoxapp.com/api/v2',
        headers: {
          'content-type': 'application/json',
          'accept': 'application/vnd.api+json',
        },
        transformRequest: [
          (data) => {
            if ('string' === typeof data) return data
            return JSON.stringify(data)
          },
        ],
        validateStatus: () => true,
      })
      const nononceResponse = await authclient.get('/authentications/new', {
        headers: {
          'X-Client-Api-Key': key,
        },
      })
      if (nononceResponse.status !== 200 || !nononceResponse.data.data) {
        const e = new DocFoxAuthenticationError(
          'Failed to request authentication nonce',
          nononceResponse
        )
        bus.emit('error', e)
        throw e
      }
      const hmac = crypto.createHmac('sha256', secret)
      hmac.update(nononceResponse.data.data.attributes.nonce)
      const signature = hmac.digest('hex')
      const jwtResponse = await authclient.get('/tokens/new', {
        headers: {
          'X-Client-Api-Key': key,
          'X-Client-Signature': signature,
        },
      })
      if (jwtResponse.status !== 200 || !jwtResponse.data.data) {
        const e = new DocFoxAuthenticationError('Failed to request JWT', jwtResponse)
        bus.emit('error', e)
        throw e
      }
      bus.emit('custom-authenticator', true)
      return {
        token: jwtResponse.data.data.attributes.token,
        expiresAt: DateTime.fromISO(jwtResponse.data.data.attributes.expires_at),
      }
    })
    const {
      status,
      data,
      config: { metadata },
    } = await client.listUser({
      per_page: 1,
    })
    assert.equal(status, 200, inspect(data, false, 10, true))
    assert.isObject(data)
    assert.isArray(data.data)
    assert.isObject(data.meta)
    assert.isTrue(metadata.reauthenticated)
    try {
      const uca = await usedCustomAuthenticator
      assert.isTrue(uca)
    } catch (e) {
      assert.fail(e)
    }
  }).timeout(10000)
  test('Can be used in an authentication function', async ({ assert }) => {
    const client = new DocFoxClient(process.env.KEY!, process.env.SECRET!)
    const nonceResponse = await client.requestAuthentication({
      'X-Client-Api-Key': process.env.KEY!,
    })
    assert.equal(
      nonceResponse.status,
      200,
      'Failed to request authentication nonce' + inspect(nonceResponse.data, false, 10, true)
    )
    assert.isObject(nonceResponse.data, 'Failed to request authentication nonce')
    assert.isObject(
      (nonceResponse.data as OperationMethodResponses['requestAuthentication']['$200']).data,
      'Failed to request authentication nonce'
    )
    const hmac = crypto.createHmac('sha256', process.env.SECRET!)
    hmac.update(
      (nonceResponse.data as OperationMethodResponses['requestAuthentication']['$200']).data
        .attributes.nonce
    )
    const signature = hmac.digest('hex')
    const jwtResponse = await client.requestToken({
      'X-Client-Api-Key': process.env.KEY!,
      'X-Client-Signature': signature,
    })
    assert.equal(
      jwtResponse.status,
      200,
      'Failed to request authentication nonce' + inspect(jwtResponse.data, false, 10, true)
    )
    assert.isObject(jwtResponse.data, 'Failed to request authentication nonce')
    assert.isObject(
      (jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data,
      'Failed to request authentication nonce'
    )
    const token = (jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data
      .attributes.token
    const expiresAt = DateTime.fromISO(
      (jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data.attributes
        .expires_at
    )
    assert.isString(token)
    assert.isTrue(expiresAt.isValid)
  }).timeout(10000)
})
