import type { AxiosResponse, InternalAxiosRequestConfig } from 'axios'
import type { AxiosRequestConfig, OpenAPIClient, Parameters } from 'openapi-client-axios'

export interface InternalAxiosRequestConfigWithMetaData<T = any>
  extends InternalAxiosRequestConfig<T> {
  metadata: Record<string, any>
}

export interface AxiosResponseWithMetaData<T = any, D = any> extends AxiosResponse<T, D> {
  config: InternalAxiosRequestConfigWithMetaData<D>
  duration?: number
}

export declare type OperationResponse<Response> = Promise<AxiosResponseWithMetaData<Response>>

export declare namespace Components {
  namespace Schemas {
    export type Phone = string
    export type Email = string
    export interface Address {
      country: string
      address_line_one: string
    }
    export type LastNames = string
    export type FirstNames = string
  }
}

export declare namespace Paths {
  namespace RequestAuthentication {
    export interface HeaderParameters {
      'X-Client-Api-Key': Parameters.XClientApiKey
    }
    export interface PathParameters {}
    export interface QueryParameters {}
    export interface OperationParameters {
      'X-Client-Api-Key': Parameters.XClientApiKey
    }
    namespace Parameters {
      export type XClientApiKey = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace RequestToken {
    export interface HeaderParameters {
      'X-Client-Signature': Parameters.XClientSignature
      'X-Client-Api-Key': Parameters.XClientApiKey
    }
    export interface PathParameters {}
    export interface QueryParameters {}
    export interface OperationParameters {
      'X-Client-Signature': Parameters.XClientSignature
      'X-Client-Api-Key': Parameters.XClientApiKey
    }
    namespace Parameters {
      export type XClientSignature = string
      export type XClientApiKey = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      filter: Parameters.Filter
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      filter: Parameters.Filter
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type Filter = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {}
    export interface OperationParameters {}
    namespace Parameters {}
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      include: Parameters.Include
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace UpdateKycApplicationApprove {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateKycApplicationUnapprove {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateKycApplicationTransfer {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ArchiveKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $204 {
        errors: { title: string; status: string; detail: string }
      }
      export interface $400 {
        errors: any[]
      }
    }
  }
  namespace UnarchiveKycApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationStatusSummarie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationProfile {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetProfile {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      include: Parameters.Include
    }
    namespace Parameters {
      export type ProfileId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileName {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateProfileName {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      name_id: Parameters.NameId
    }
    export interface OperationParameters {
      name_id: Parameters.NameId
      profile_id: Parameters.ProfileId
    }
    namespace Parameters {
      export type NameId = string
      export type ProfileId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetName {
    export interface HeaderParameters {}
    export interface PathParameters {
      name_id: Parameters.NameId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      name_id: Parameters.NameId
    }
    namespace Parameters {
      export type NameId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteName {
    export interface HeaderParameters {}
    export interface PathParameters {
      name_id: Parameters.NameId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      name_id: Parameters.NameId
    }
    namespace Parameters {
      export type NameId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace UpdateName {
    export interface HeaderParameters {}
    export interface PathParameters {
      name_id: Parameters.NameId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      name_id: Parameters.NameId
    }
    namespace Parameters {
      export type NameId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileNumber {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateProfileNumber {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      number_id: Parameters.NumberId
    }
    export interface OperationParameters {
      number_id: Parameters.NumberId
      profile_id: Parameters.ProfileId
    }
    namespace Parameters {
      export type NumberId = string
      export type ProfileId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetNumber {
    export interface HeaderParameters {}
    export interface PathParameters {
      number_id: Parameters.NumberId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      number_id: Parameters.NumberId
    }
    namespace Parameters {
      export type NumberId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteNumber {
    export interface HeaderParameters {}
    export interface PathParameters {
      number_id: Parameters.NumberId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      number_id: Parameters.NumberId
    }
    namespace Parameters {
      export type NumberId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace UpdateNumber {
    export interface HeaderParameters {}
    export interface PathParameters {
      number_id: Parameters.NumberId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      number_id: Parameters.NumberId
    }
    namespace Parameters {
      export type NumberId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileContact {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateProfileContact {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      contact_id: Parameters.ContactId
    }
    export interface OperationParameters {
      contact_id: Parameters.ContactId
      profile_id: Parameters.ProfileId
    }
    namespace Parameters {
      export type ContactId = string
      export type ProfileId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetContact {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_id: Parameters.ContactId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      contact_id: Parameters.ContactId
    }
    namespace Parameters {
      export type ContactId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteContact {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_id: Parameters.ContactId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      contact_id: Parameters.ContactId
    }
    namespace Parameters {
      export type ContactId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace UpdateContact {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_id: Parameters.ContactId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      contact_id: Parameters.ContactId
    }
    namespace Parameters {
      export type ContactId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileAddresse {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateProfileAddresse {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      address_id: Parameters.AddressId
    }
    export interface OperationParameters {
      address_id: Parameters.AddressId
      profile_id: Parameters.ProfileId
    }
    namespace Parameters {
      export type AddressId = string
      export type ProfileId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAddresse {
    export interface HeaderParameters {}
    export interface PathParameters {
      address_id: Parameters.AddressId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      address_id: Parameters.AddressId
    }
    namespace Parameters {
      export type AddressId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteAddresse {
    export interface HeaderParameters {}
    export interface PathParameters {
      address_id: Parameters.AddressId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      address_id: Parameters.AddressId
    }
    namespace Parameters {
      export type AddressId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace UpdateAddresse {
    export interface HeaderParameters {}
    export interface PathParameters {
      address_id: Parameters.AddressId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      address_id: Parameters.AddressId
    }
    namespace Parameters {
      export type AddressId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileAdditionalDetail {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateProfileAdditionalDetail {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    export interface OperationParameters {
      additional_detail_id: Parameters.AdditionalDetailId
      profile_id: Parameters.ProfileId
    }
    namespace Parameters {
      export type AdditionalDetailId = string
      export type ProfileId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAdditionalDetail {
    export interface HeaderParameters {}
    export interface PathParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    namespace Parameters {
      export type AdditionalDetailId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteAdditionalDetail {
    export interface HeaderParameters {}
    export interface PathParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    namespace Parameters {
      export type AdditionalDetailId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace UpdateAdditionalDetail {
    export interface HeaderParameters {}
    export interface PathParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      additional_detail_id: Parameters.AdditionalDetailId
    }
    namespace Parameters {
      export type AdditionalDetailId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListContactInvitationToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_information_id: Parameters.ContactInformationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      contact_information_id: Parameters.ContactInformationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ContactInformationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateContactInvitationToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_information_id: Parameters.ContactInformationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      contact_information_id: Parameters.ContactInformationId
    }
    namespace Parameters {
      export type ContactInformationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace GetInvitationToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      invitation_token_id: Parameters.InvitationTokenId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      invitation_token_id: Parameters.InvitationTokenId
    }
    namespace Parameters {
      export type InvitationTokenId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateInvitationToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      invitation_token_id: Parameters.InvitationTokenId
    }
    export interface QueryParameters {
      contact_information_id: Parameters.ContactInformationId
    }
    export interface OperationParameters {
      contact_information_id: Parameters.ContactInformationId
      invitation_token_id: Parameters.InvitationTokenId
    }
    namespace Parameters {
      export type ContactInformationId = string
      export type InvitationTokenId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
    }
  }
  namespace ListContactDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_information_id: Parameters.ContactInformationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      contact_information_id: Parameters.ContactInformationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ContactInformationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_permission_id: Parameters.DataRequirementPermissionId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      data_requirement_permission_id: Parameters.DataRequirementPermissionId
    }
    namespace Parameters {
      export type DataRequirementPermissionId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_permission_id: Parameters.DataRequirementPermissionId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      data_requirement_permission_id: Parameters.DataRequirementPermissionId
    }
    namespace Parameters {
      export type DataRequirementPermissionId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace CreateDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {}
    export interface OperationParameters {}
    namespace Parameters {}
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace ListDataRequirementDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      data_requirement_id: Parameters.DataRequirementId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type DataRequirementId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateDataRequirementDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    namespace Parameters {
      export type DataRequirementId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace CreateContactDataRequirementPermission {
    export interface HeaderParameters {}
    export interface PathParameters {
      contact_id: Parameters.ContactId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      contact_id: Parameters.ContactId
    }
    namespace Parameters {
      export type ContactId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace ListKycApplicationDataRequirement {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      forms: Parameters.Forms
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      forms: Parameters.Forms
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Forms = boolean
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDataRequirement {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      data_requirement_id: Parameters.DataRequirementId
      include: Parameters.Include
    }
    namespace Parameters {
      export type DataRequirementId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycEntityTemplate {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetKycEntityTemplate {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
    }
    namespace Parameters {
      export type KycEntityTemplateId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationDataComparison {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDataComparison {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_comparison_id: Parameters.DataComparisonId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      data_comparison_id: Parameters.DataComparisonId
    }
    namespace Parameters {
      export type DataComparisonId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProfileDataComparison {
    export interface HeaderParameters {}
    export interface PathParameters {
      profile_id: Parameters.ProfileId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      profile_id: Parameters.ProfileId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProfileId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListDataRequirementDataComparison {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      data_requirement_id: Parameters.DataRequirementId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type DataRequirementId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListDataComparisonComparisonResult {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_comparison_id: Parameters.DataComparisonId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      data_comparison_id: Parameters.DataComparisonId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type DataComparisonId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetComparisonResult {
    export interface HeaderParameters {}
    export interface PathParameters {
      comparison_result_id: Parameters.ComparisonResultId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      comparison_result_id: Parameters.ComparisonResultId
    }
    namespace Parameters {
      export type ComparisonResultId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListDataRequirementEvidenceSubmission {
    export interface HeaderParameters {}
    export interface PathParameters {
      data_requirement_id: Parameters.DataRequirementId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      data_requirement_id: Parameters.DataRequirementId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type DataRequirementId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetEvidenceSubmission {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      include: Parameters.Include
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateEvidenceSubmission {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      include: Parameters.Include
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        errors: any[]
      }
    }
  }
  namespace UpdateEvidenceSubmissionReject {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      include: Parameters.Include
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateEvidenceSubmissionApprove {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      include: Parameters.Include
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateEvidenceSubmissionReplace {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      include: Parameters.Include
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListEvidenceSubmissionQualitativeCheck {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetQualitativeCheck {
    export interface HeaderParameters {}
    export interface PathParameters {
      qualitative_check_id: Parameters.QualitativeCheckId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      qualitative_check_id: Parameters.QualitativeCheckId
    }
    namespace Parameters {
      export type QualitativeCheckId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListEvidenceSubmissionRejectionReason {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetRejectionReason {
    export interface HeaderParameters {}
    export interface PathParameters {
      rejection_reason_id: Parameters.RejectionReasonId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      rejection_reason_id: Parameters.RejectionReasonId
    }
    namespace Parameters {
      export type RejectionReasonId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListEvidenceSubmissionDocument {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateEvidenceSubmissionDocument {
    export interface HeaderParameters {}
    export interface PathParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      evidence_submission_id: Parameters.EvidenceSubmissionId
    }
    namespace Parameters {
      export type EvidenceSubmissionId = string
    }
    namespace Responses {
      export interface $201 {
        data: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace GetDocument {
    export interface HeaderParameters {}
    export interface PathParameters {
      document_id: Parameters.DocumentId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      document_id: Parameters.DocumentId
      include: Parameters.Include
    }
    namespace Parameters {
      export type DocumentId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListDocumentDocumentToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      document_id: Parameters.DocumentId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      document_id: Parameters.DocumentId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type DocumentId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateDocumentDocumentToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      document_id: Parameters.DocumentId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      document_id: Parameters.DocumentId
    }
    namespace Parameters {
      export type DocumentId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDocumentToken {
    export interface HeaderParameters {}
    export interface PathParameters {
      document_token_id: Parameters.DocumentTokenId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      document_token_id: Parameters.DocumentTokenId
    }
    namespace Parameters {
      export type DocumentTokenId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDocumentFileDownload {
    export interface HeaderParameters {}
    export interface PathParameters {
      document_token_id: Parameters.DocumentTokenId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      document_token_id: Parameters.DocumentTokenId
    }
    namespace Parameters {
      export type DocumentTokenId = Record<string, any>
    }
    namespace Responses {
      export interface $302 {}
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: any[]
      }
      export interface $410 {
        errors: any[]
      }
    }
  }
  namespace GetForm {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_id: Parameters.FormId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      form_id: Parameters.FormId
      include: Parameters.Include
    }
    namespace Parameters {
      export type FormId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetFormFormData {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_id: Parameters.FormId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      form_id: Parameters.FormId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type FormId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateForm {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_id: Parameters.FormId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      form_id: Parameters.FormId
    }
    namespace Parameters {
      export type FormId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetFormDataById {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_datum_id: Parameters.FormDatumId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      form_datum_id: Parameters.FormDatumId
    }
    namespace Parameters {
      export type FormDatumId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationRelatedEntitie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
    }
  }
  namespace CreateKycApplicationRelatedEntitie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $409 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace GetRelatedEntitie {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    export interface QueryParameters {
      include: Parameters.Include
    }
    export interface OperationParameters {
      related_entity_id: Parameters.RelatedEntityId
      include: Parameters.Include
    }
    namespace Parameters {
      export type RelatedEntityId = string
      export type Include = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteRelatedEntitie {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    namespace Parameters {
      export type RelatedEntityId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $422 {
        errors: any[]
      }
    }
  }
  namespace ListKycApplicationPdfReportSummarie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateKycApplicationPdfReportSummarie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $429 {
        errors: any[]
      }
    }
  }
  namespace GetPdfReportSummarie {
    export interface HeaderParameters {}
    export interface PathParameters {
      pdf_report_summary_id: Parameters.PdfReportSummaryId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      pdf_report_summary_id: Parameters.PdfReportSummaryId
    }
    namespace Parameters {
      export type PdfReportSummaryId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetPdfReportSummarieDownload {
    export interface HeaderParameters {}
    export interface PathParameters {
      pdf_summary_report_id: Parameters.PdfSummaryReportId
    }
    export interface QueryParameters {
      pdf_report_summary_id: Parameters.PdfReportSummaryId
    }
    export interface OperationParameters {
      pdf_report_summary_id: Parameters.PdfReportSummaryId
      pdf_summary_report_id: Parameters.PdfSummaryReportId
    }
    namespace Parameters {
      export type PdfReportSummaryId = string
      export type PdfSummaryReportId = string
    }
    namespace Responses {
      export interface $302 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $406 {
        errors: any[]
      }
      export interface $409 {
        errors: any[]
      }
    }
  }
  namespace ListUser {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetUser {
    export interface HeaderParameters {}
    export interface PathParameters {
      user_id: Parameters.UserId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      user_id: Parameters.UserId
    }
    namespace Parameters {
      export type UserId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetRiskRatingEvaluation {
    export interface HeaderParameters {}
    export interface PathParameters {
      risk_rating_evaluation_id: Parameters.RiskRatingEvaluationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      risk_rating_evaluation_id: Parameters.RiskRatingEvaluationId
    }
    namespace Parameters {
      export type RiskRatingEvaluationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationRiskRatingEvaluation {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationRiskRating {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateKycApplicationRiskRating {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetKycApplicationRiskRating {
    export interface HeaderParameters {}
    export interface PathParameters {
      evaluation_id: Parameters.EvaluationId
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      evaluation_id: Parameters.EvaluationId
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type EvaluationId = string
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateKycApplicationRiskRating {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
      evaluation_id: Parameters.EvaluationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      evaluation_id: Parameters.EvaluationId
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type EvaluationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationRiskRatingsQuestion {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: Record<string, any>
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace GetBankAccountSchema {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationBankAccount {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateKycApplicationBankAccount {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
        included: any[]
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetBankAccount {
    export interface HeaderParameters {}
    export interface PathParameters {
      bank_account_id: Parameters.BankAccountId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      bank_account_id: Parameters.BankAccountId
    }
    namespace Parameters {
      export type BankAccountId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetBankAccountVerification {
    export interface HeaderParameters {}
    export interface PathParameters {
      bank_account_verification_id: Parameters.BankAccountVerificationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      bank_account_verification_id: Parameters.BankAccountVerificationId
    }
    namespace Parameters {
      export type BankAccountVerificationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateBankAccountVerification {
    export interface HeaderParameters {}
    export interface PathParameters {
      bank_account_verification_id: Parameters.BankAccountVerificationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      bank_account_verification_id: Parameters.BankAccountVerificationId
    }
    namespace Parameters {
      export type BankAccountVerificationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListBankAccountBankAccountVerification {
    export interface HeaderParameters {}
    export interface PathParameters {
      bank_account_id: Parameters.BankAccountId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      bank_account_id: Parameters.BankAccountId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type BankAccountId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationConsumerReportQuerie {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetConsumerReportQuerie {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetConsumerReportQuerieDownload {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $302 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $406 {
        errors: any[]
      }
      export interface $409 {
        errors: any[]
      }
    }
  }
  namespace CreateOnboardingInvitation {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {}
    export interface OperationParameters {}
    namespace Parameters {}
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateOnboardingInvitation {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListFormFormPdfExport {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_id: Parameters.FormId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      form_id: Parameters.FormId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type FormId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateFormFormPdfExport {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_id: Parameters.FormId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      form_id: Parameters.FormId
    }
    namespace Parameters {
      export type FormId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $429 {
        errors: any[]
      }
    }
  }
  namespace GetFormPdfExport {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_pdf_export_id: Parameters.FormPdfExportId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      form_pdf_export_id: Parameters.FormPdfExportId
    }
    namespace Parameters {
      export type FormPdfExportId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetFormPdfExportDownload {
    export interface HeaderParameters {}
    export interface PathParameters {
      form_pdf_export_id: Parameters.FormPdfExportId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      form_pdf_export_id: Parameters.FormPdfExportId
    }
    namespace Parameters {
      export type FormPdfExportId = string
    }
    namespace Responses {
      export interface $302 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
      export interface $406 {
        errors: any[]
      }
      export interface $409 {
        errors: any[]
      }
    }
  }
  namespace ListKycEntityTemplateDesignationTemplate {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycEntityTemplateId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetRelatedEntityDesignation {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_designation_id: Parameters.RelatedEntityDesignationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      related_entity_designation_id: Parameters.RelatedEntityDesignationId
    }
    namespace Parameters {
      export type RelatedEntityDesignationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace DeleteRelatedEntityDesignation {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_designation_id: Parameters.RelatedEntityDesignationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      related_entity_designation_id: Parameters.RelatedEntityDesignationId
    }
    namespace Parameters {
      export type RelatedEntityDesignationId = string
    }
    namespace Responses {
      export interface $204 {}
      export interface $400 {
        errors: any[]
      }
      export interface $401 {
        errors: any[]
      }
      export interface $404 {
        errors: any[]
      }
    }
  }
  namespace ListRelatedEntitieRelatedEntityDesignation {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      related_entity_id: Parameters.RelatedEntityId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type RelatedEntityId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateRelatedEntitieRelatedEntityDesignation {
    export interface HeaderParameters {}
    export interface PathParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      related_entity_id: Parameters.RelatedEntityId
    }
    namespace Parameters {
      export type RelatedEntityId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $422 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProductProductProductAddon {
    export interface HeaderParameters {}
    export interface PathParameters {
      product_id: Parameters.ProductId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      product_id: Parameters.ProductId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProductId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListAccountApplicationAccountAddon {
    export interface HeaderParameters {}
    export interface PathParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      account_application_id: Parameters.AccountApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type AccountApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAccountAddon {
    export interface HeaderParameters {}
    export interface PathParameters {
      account_addon_id: Parameters.AccountAddonId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      account_addon_id: Parameters.AccountAddonId
    }
    namespace Parameters {
      export type AccountAddonId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProductDisclosure {
    export interface HeaderParameters {}
    export interface PathParameters {
      product_id: Parameters.ProductId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      product_id: Parameters.ProductId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProductId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListDisclosure {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetDisclosure {
    export interface HeaderParameters {}
    export interface PathParameters {
      disclosure_id: Parameters.DisclosureId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      disclosure_id: Parameters.DisclosureId
    }
    namespace Parameters {
      export type DisclosureId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProduct {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetProduct {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycEntityTemplateProduct {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycEntityTemplateId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAccountApplicationAccountSummaryPdf {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $200 {
        data: Record<string, any>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycEntityTemplateDisclosure {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_entity_template_id: Parameters.KycEntityTemplateId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycEntityTemplateId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListAccountApplicationDataRequirement {
    export interface HeaderParameters {}
    export interface PathParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    export interface QueryParameters {
      forms: Parameters.Forms
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      account_application_id: Parameters.AccountApplicationId
      forms: Parameters.Forms
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type AccountApplicationId = string
      export type Forms = boolean
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListProductProductAddonsDisclosure {
    export interface HeaderParameters {}
    export interface PathParameters {}
    export interface QueryParameters {
      product_product_addon_ids: Parameters.ProductProductAddonIds
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      product_product_addon_ids: Parameters.ProductProductAddonIds
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type ProductProductAddonIds = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $403 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAccountApplicationAccountSummary {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $200 {
        data: Record<string, any>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace ListKycApplicationAccountApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
      page: Parameters.Page
      per_page: Parameters.PerPage
    }
    namespace Parameters {
      export type KycApplicationId = string
      export type Page = number
      export type PerPage = number
    }
    namespace Responses {
      export interface $200 {
        data: Array<{
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }>
        meta: Record<string, any>
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace CreateKycApplicationAccountApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      kyc_application_id: Parameters.KycApplicationId
    }
    namespace Parameters {
      export type KycApplicationId = string
    }
    namespace Responses {
      export interface $201 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace GetAccountApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    namespace Parameters {
      export type AccountApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateAccountApplication {
    export interface HeaderParameters {}
    export interface PathParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      account_application_id: Parameters.AccountApplicationId
    }
    namespace Parameters {
      export type AccountApplicationId = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
  namespace UpdateAccountApplicationDisclosureAcceptances {
    export interface HeaderParameters {}
    export interface PathParameters {
      id: Parameters.Id
    }
    export interface QueryParameters {}
    export interface OperationParameters {
      id: Parameters.Id
    }
    namespace Parameters {
      export type Id = string
    }
    namespace Responses {
      export interface $200 {
        data: {
          id: string
          type: string
          attributes: Record<string, any>
          relationships: Record<string, any>
          links: Record<string, any>
        }
      }
      export interface $400 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $401 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $404 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
      export interface $406 {
        errors: Array<{ title: string; status: string; detail: string }>
      }
    }
  }
}

export interface OperationMethods {
  'requestAuthentication'(
    parameters?: Parameters<Paths.RequestAuthentication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.RequestAuthentication.Responses.$200
    | Paths.RequestAuthentication.Responses.$400
    | Paths.RequestAuthentication.Responses.$401
  >

  'requestToken'(
    parameters?: Parameters<Paths.RequestToken.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.RequestToken.Responses.$200
    | Paths.RequestToken.Responses.$400
    | Paths.RequestToken.Responses.$401
  >

  'listKycApplication'(
    parameters?: Parameters<Paths.ListKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplication.Responses.$200
    | Paths.ListKycApplication.Responses.$400
    | Paths.ListKycApplication.Responses.$401
    | Paths.ListKycApplication.Responses.$403
  >

  'createKycApplication'(
    parameters?: Parameters<Paths.CreateKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplication.Responses.$201
    | Paths.CreateKycApplication.Responses.$400
    | Paths.CreateKycApplication.Responses.$401
    | Paths.CreateKycApplication.Responses.$406
    | Paths.CreateKycApplication.Responses.$422
  >

  'getKycApplication'(
    parameters?: Parameters<Paths.GetKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetKycApplication.Responses.$200
    | Paths.GetKycApplication.Responses.$400
    | Paths.GetKycApplication.Responses.$401
    | Paths.GetKycApplication.Responses.$404
  >

  'deleteKycApplication'(
    parameters?: Parameters<Paths.DeleteKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteKycApplication.Responses.$204
    | Paths.DeleteKycApplication.Responses.$400
    | Paths.DeleteKycApplication.Responses.$401
    | Paths.DeleteKycApplication.Responses.$404
  >

  'updateKycApplicationApprove'(
    parameters?:
      | Parameters<Paths.UpdateKycApplicationApprove.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateKycApplicationApprove.Responses.$200
    | Paths.UpdateKycApplicationApprove.Responses.$400
    | Paths.UpdateKycApplicationApprove.Responses.$401
    | Paths.UpdateKycApplicationApprove.Responses.$404
    | Paths.UpdateKycApplicationApprove.Responses.$406
    | Paths.UpdateKycApplicationApprove.Responses.$422
  >

  'updateKycApplicationUnapprove'(
    parameters?:
      | Parameters<Paths.UpdateKycApplicationUnapprove.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateKycApplicationUnapprove.Responses.$200
    | Paths.UpdateKycApplicationUnapprove.Responses.$400
    | Paths.UpdateKycApplicationUnapprove.Responses.$401
    | Paths.UpdateKycApplicationUnapprove.Responses.$404
    | Paths.UpdateKycApplicationUnapprove.Responses.$406
    | Paths.UpdateKycApplicationUnapprove.Responses.$422
  >

  'updateKycApplicationTransfer'(
    parameters?:
      | Parameters<Paths.UpdateKycApplicationTransfer.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateKycApplicationTransfer.Responses.$200
    | Paths.UpdateKycApplicationTransfer.Responses.$400
    | Paths.UpdateKycApplicationTransfer.Responses.$401
    | Paths.UpdateKycApplicationTransfer.Responses.$404
    | Paths.UpdateKycApplicationTransfer.Responses.$406
    | Paths.UpdateKycApplicationTransfer.Responses.$422
  >

  'archiveKycApplication'(
    parameters?: Parameters<Paths.ArchiveKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    Paths.ArchiveKycApplication.Responses.$204 | Paths.ArchiveKycApplication.Responses.$400
  >

  'unarchiveKycApplication'(
    parameters?: Parameters<Paths.UnarchiveKycApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    Paths.UnarchiveKycApplication.Responses.$200 | Paths.UnarchiveKycApplication.Responses.$400
  >

  'listKycApplicationStatusSummarie'(
    parameters?:
      | Parameters<Paths.ListKycApplicationStatusSummarie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationStatusSummarie.Responses.$200
    | Paths.ListKycApplicationStatusSummarie.Responses.$400
    | Paths.ListKycApplicationStatusSummarie.Responses.$401
    | Paths.ListKycApplicationStatusSummarie.Responses.$404
  >

  'listKycApplicationProfile'(
    parameters?: Parameters<Paths.ListKycApplicationProfile.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationProfile.Responses.$200
    | Paths.ListKycApplicationProfile.Responses.$400
    | Paths.ListKycApplicationProfile.Responses.$401
    | Paths.ListKycApplicationProfile.Responses.$404
  >

  'getProfile'(
    parameters?: Parameters<Paths.GetProfile.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetProfile.Responses.$200
    | Paths.GetProfile.Responses.$400
    | Paths.GetProfile.Responses.$401
    | Paths.GetProfile.Responses.$404
  >

  'listProfileName'(
    parameters?: Parameters<Paths.ListProfileName.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileName.Responses.$200
    | Paths.ListProfileName.Responses.$400
    | Paths.ListProfileName.Responses.$401
    | Paths.ListProfileName.Responses.$404
  >

  'createProfileName'(
    parameters?: Parameters<Paths.CreateProfileName.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateProfileName.Responses.$201
    | Paths.CreateProfileName.Responses.$400
    | Paths.CreateProfileName.Responses.$401
    | Paths.CreateProfileName.Responses.$404
    | Paths.CreateProfileName.Responses.$406
    | Paths.CreateProfileName.Responses.$422
  >

  'getName'(
    parameters?: Parameters<Paths.GetName.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetName.Responses.$200
    | Paths.GetName.Responses.$400
    | Paths.GetName.Responses.$401
    | Paths.GetName.Responses.$404
  >

  'deleteName'(
    parameters?: Parameters<Paths.DeleteName.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteName.Responses.$204
    | Paths.DeleteName.Responses.$400
    | Paths.DeleteName.Responses.$401
    | Paths.DeleteName.Responses.$404
  >

  'updateName'(
    parameters?: Parameters<Paths.UpdateName.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateName.Responses.$200
    | Paths.UpdateName.Responses.$400
    | Paths.UpdateName.Responses.$401
    | Paths.UpdateName.Responses.$404
    | Paths.UpdateName.Responses.$406
    | Paths.UpdateName.Responses.$422
  >

  'listProfileNumber'(
    parameters?: Parameters<Paths.ListProfileNumber.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileNumber.Responses.$200
    | Paths.ListProfileNumber.Responses.$400
    | Paths.ListProfileNumber.Responses.$401
    | Paths.ListProfileNumber.Responses.$404
  >

  'createProfileNumber'(
    parameters?: Parameters<Paths.CreateProfileNumber.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateProfileNumber.Responses.$201
    | Paths.CreateProfileNumber.Responses.$400
    | Paths.CreateProfileNumber.Responses.$401
    | Paths.CreateProfileNumber.Responses.$404
    | Paths.CreateProfileNumber.Responses.$406
    | Paths.CreateProfileNumber.Responses.$422
  >

  'getNumber'(
    parameters?: Parameters<Paths.GetNumber.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetNumber.Responses.$200
    | Paths.GetNumber.Responses.$400
    | Paths.GetNumber.Responses.$401
    | Paths.GetNumber.Responses.$404
  >

  'deleteNumber'(
    parameters?: Parameters<Paths.DeleteNumber.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteNumber.Responses.$204
    | Paths.DeleteNumber.Responses.$400
    | Paths.DeleteNumber.Responses.$401
    | Paths.DeleteNumber.Responses.$404
    | Paths.DeleteNumber.Responses.$422
  >

  'updateNumber'(
    parameters?: Parameters<Paths.UpdateNumber.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateNumber.Responses.$200
    | Paths.UpdateNumber.Responses.$400
    | Paths.UpdateNumber.Responses.$401
    | Paths.UpdateNumber.Responses.$404
    | Paths.UpdateNumber.Responses.$406
    | Paths.UpdateNumber.Responses.$422
  >

  'listProfileContact'(
    parameters?: Parameters<Paths.ListProfileContact.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileContact.Responses.$200
    | Paths.ListProfileContact.Responses.$400
    | Paths.ListProfileContact.Responses.$401
    | Paths.ListProfileContact.Responses.$404
  >

  'createProfileContact'(
    parameters?: Parameters<Paths.CreateProfileContact.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateProfileContact.Responses.$201
    | Paths.CreateProfileContact.Responses.$400
    | Paths.CreateProfileContact.Responses.$401
    | Paths.CreateProfileContact.Responses.$404
    | Paths.CreateProfileContact.Responses.$406
    | Paths.CreateProfileContact.Responses.$422
  >

  'getContact'(
    parameters?: Parameters<Paths.GetContact.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetContact.Responses.$200
    | Paths.GetContact.Responses.$400
    | Paths.GetContact.Responses.$401
    | Paths.GetContact.Responses.$404
  >

  'deleteContact'(
    parameters?: Parameters<Paths.DeleteContact.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteContact.Responses.$204
    | Paths.DeleteContact.Responses.$400
    | Paths.DeleteContact.Responses.$401
    | Paths.DeleteContact.Responses.$404
  >

  'updateContact'(
    parameters?: Parameters<Paths.UpdateContact.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateContact.Responses.$200
    | Paths.UpdateContact.Responses.$400
    | Paths.UpdateContact.Responses.$401
    | Paths.UpdateContact.Responses.$404
    | Paths.UpdateContact.Responses.$406
    | Paths.UpdateContact.Responses.$422
  >

  'listProfileAddresse'(
    parameters?: Parameters<Paths.ListProfileAddresse.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileAddresse.Responses.$200
    | Paths.ListProfileAddresse.Responses.$400
    | Paths.ListProfileAddresse.Responses.$401
    | Paths.ListProfileAddresse.Responses.$404
  >

  'createProfileAddresse'(
    parameters?: Parameters<Paths.CreateProfileAddresse.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateProfileAddresse.Responses.$201
    | Paths.CreateProfileAddresse.Responses.$400
    | Paths.CreateProfileAddresse.Responses.$401
    | Paths.CreateProfileAddresse.Responses.$404
    | Paths.CreateProfileAddresse.Responses.$406
    | Paths.CreateProfileAddresse.Responses.$422
  >

  'getAddresse'(
    parameters?: Parameters<Paths.GetAddresse.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAddresse.Responses.$200
    | Paths.GetAddresse.Responses.$400
    | Paths.GetAddresse.Responses.$401
    | Paths.GetAddresse.Responses.$404
  >

  'deleteAddresse'(
    parameters?: Parameters<Paths.DeleteAddresse.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteAddresse.Responses.$204
    | Paths.DeleteAddresse.Responses.$400
    | Paths.DeleteAddresse.Responses.$401
    | Paths.DeleteAddresse.Responses.$404
  >

  'updateAddresse'(
    parameters?: Parameters<Paths.UpdateAddresse.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateAddresse.Responses.$200
    | Paths.UpdateAddresse.Responses.$400
    | Paths.UpdateAddresse.Responses.$401
    | Paths.UpdateAddresse.Responses.$404
    | Paths.UpdateAddresse.Responses.$406
    | Paths.UpdateAddresse.Responses.$422
  >

  'listProfileAdditionalDetail'(
    parameters?:
      | Parameters<Paths.ListProfileAdditionalDetail.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileAdditionalDetail.Responses.$200
    | Paths.ListProfileAdditionalDetail.Responses.$400
    | Paths.ListProfileAdditionalDetail.Responses.$401
    | Paths.ListProfileAdditionalDetail.Responses.$404
  >

  'createProfileAdditionalDetail'(
    parameters?:
      | Parameters<Paths.CreateProfileAdditionalDetail.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateProfileAdditionalDetail.Responses.$201
    | Paths.CreateProfileAdditionalDetail.Responses.$400
    | Paths.CreateProfileAdditionalDetail.Responses.$401
    | Paths.CreateProfileAdditionalDetail.Responses.$404
    | Paths.CreateProfileAdditionalDetail.Responses.$406
    | Paths.CreateProfileAdditionalDetail.Responses.$422
  >

  'getAdditionalDetail'(
    parameters?: Parameters<Paths.GetAdditionalDetail.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAdditionalDetail.Responses.$200
    | Paths.GetAdditionalDetail.Responses.$400
    | Paths.GetAdditionalDetail.Responses.$401
    | Paths.GetAdditionalDetail.Responses.$404
  >

  'deleteAdditionalDetail'(
    parameters?: Parameters<Paths.DeleteAdditionalDetail.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteAdditionalDetail.Responses.$204
    | Paths.DeleteAdditionalDetail.Responses.$400
    | Paths.DeleteAdditionalDetail.Responses.$401
    | Paths.DeleteAdditionalDetail.Responses.$404
    | Paths.DeleteAdditionalDetail.Responses.$422
  >

  'updateAdditionalDetail'(
    parameters?: Parameters<Paths.UpdateAdditionalDetail.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateAdditionalDetail.Responses.$200
    | Paths.UpdateAdditionalDetail.Responses.$400
    | Paths.UpdateAdditionalDetail.Responses.$401
    | Paths.UpdateAdditionalDetail.Responses.$404
    | Paths.UpdateAdditionalDetail.Responses.$406
    | Paths.UpdateAdditionalDetail.Responses.$422
  >

  'listContactInvitationToken'(
    parameters?:
      | Parameters<Paths.ListContactInvitationToken.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListContactInvitationToken.Responses.$200
    | Paths.ListContactInvitationToken.Responses.$400
    | Paths.ListContactInvitationToken.Responses.$401
    | Paths.ListContactInvitationToken.Responses.$404
  >

  'createContactInvitationToken'(
    parameters?:
      | Parameters<Paths.CreateContactInvitationToken.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateContactInvitationToken.Responses.$201
    | Paths.CreateContactInvitationToken.Responses.$400
    | Paths.CreateContactInvitationToken.Responses.$401
    | Paths.CreateContactInvitationToken.Responses.$404
    | Paths.CreateContactInvitationToken.Responses.$406
    | Paths.CreateContactInvitationToken.Responses.$409
    | Paths.CreateContactInvitationToken.Responses.$422
  >

  'getInvitationToken'(
    parameters?: Parameters<Paths.GetInvitationToken.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetInvitationToken.Responses.$200
    | Paths.GetInvitationToken.Responses.$400
    | Paths.GetInvitationToken.Responses.$401
    | Paths.GetInvitationToken.Responses.$404
  >

  'updateInvitationToken'(
    parameters?: Parameters<Paths.UpdateInvitationToken.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateInvitationToken.Responses.$200
    | Paths.UpdateInvitationToken.Responses.$400
    | Paths.UpdateInvitationToken.Responses.$401
    | Paths.UpdateInvitationToken.Responses.$404
    | Paths.UpdateInvitationToken.Responses.$406
    | Paths.UpdateInvitationToken.Responses.$409
  >

  'listContactDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.ListContactDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListContactDataRequirementPermission.Responses.$200
    | Paths.ListContactDataRequirementPermission.Responses.$400
    | Paths.ListContactDataRequirementPermission.Responses.$401
    | Paths.ListContactDataRequirementPermission.Responses.$404
  >

  'getDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.GetDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDataRequirementPermission.Responses.$200
    | Paths.GetDataRequirementPermission.Responses.$400
    | Paths.GetDataRequirementPermission.Responses.$401
    | Paths.GetDataRequirementPermission.Responses.$404
  >

  'deleteDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.DeleteDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteDataRequirementPermission.Responses.$204
    | Paths.DeleteDataRequirementPermission.Responses.$400
    | Paths.DeleteDataRequirementPermission.Responses.$401
    | Paths.DeleteDataRequirementPermission.Responses.$404
  >

  'createDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.CreateDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateDataRequirementPermission.Responses.$201
    | Paths.CreateDataRequirementPermission.Responses.$400
    | Paths.CreateDataRequirementPermission.Responses.$401
    | Paths.CreateDataRequirementPermission.Responses.$404
    | Paths.CreateDataRequirementPermission.Responses.$406
    | Paths.CreateDataRequirementPermission.Responses.$409
    | Paths.CreateDataRequirementPermission.Responses.$422
  >

  'listDataRequirementDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.ListDataRequirementDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDataRequirementDataRequirementPermission.Responses.$200
    | Paths.ListDataRequirementDataRequirementPermission.Responses.$400
    | Paths.ListDataRequirementDataRequirementPermission.Responses.$401
    | Paths.ListDataRequirementDataRequirementPermission.Responses.$404
  >

  'createDataRequirementDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.CreateDataRequirementDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$201
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$400
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$401
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$404
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$406
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$409
    | Paths.CreateDataRequirementDataRequirementPermission.Responses.$422
  >

  'createContactDataRequirementPermission'(
    parameters?:
      | Parameters<Paths.CreateContactDataRequirementPermission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateContactDataRequirementPermission.Responses.$201
    | Paths.CreateContactDataRequirementPermission.Responses.$400
    | Paths.CreateContactDataRequirementPermission.Responses.$401
    | Paths.CreateContactDataRequirementPermission.Responses.$404
    | Paths.CreateContactDataRequirementPermission.Responses.$406
    | Paths.CreateContactDataRequirementPermission.Responses.$409
    | Paths.CreateContactDataRequirementPermission.Responses.$422
  >

  'listKycApplicationDataRequirement'(
    parameters?:
      | Parameters<Paths.ListKycApplicationDataRequirement.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationDataRequirement.Responses.$200
    | Paths.ListKycApplicationDataRequirement.Responses.$400
    | Paths.ListKycApplicationDataRequirement.Responses.$401
    | Paths.ListKycApplicationDataRequirement.Responses.$404
  >

  'getDataRequirement'(
    parameters?: Parameters<Paths.GetDataRequirement.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDataRequirement.Responses.$200
    | Paths.GetDataRequirement.Responses.$400
    | Paths.GetDataRequirement.Responses.$401
    | Paths.GetDataRequirement.Responses.$404
  >

  'listKycEntityTemplate'(
    parameters?: Parameters<Paths.ListKycEntityTemplate.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycEntityTemplate.Responses.$200
    | Paths.ListKycEntityTemplate.Responses.$400
    | Paths.ListKycEntityTemplate.Responses.$401
    | Paths.ListKycEntityTemplate.Responses.$403
  >

  'getKycEntityTemplate'(
    parameters?: Parameters<Paths.GetKycEntityTemplate.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetKycEntityTemplate.Responses.$200
    | Paths.GetKycEntityTemplate.Responses.$400
    | Paths.GetKycEntityTemplate.Responses.$401
    | Paths.GetKycEntityTemplate.Responses.$404
  >

  'listKycApplicationDataComparison'(
    parameters?:
      | Parameters<Paths.ListKycApplicationDataComparison.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationDataComparison.Responses.$200
    | Paths.ListKycApplicationDataComparison.Responses.$400
    | Paths.ListKycApplicationDataComparison.Responses.$401
    | Paths.ListKycApplicationDataComparison.Responses.$404
  >

  'getDataComparison'(
    parameters?: Parameters<Paths.GetDataComparison.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDataComparison.Responses.$200
    | Paths.GetDataComparison.Responses.$400
    | Paths.GetDataComparison.Responses.$401
    | Paths.GetDataComparison.Responses.$404
  >

  'listProfileDataComparison'(
    parameters?: Parameters<Paths.ListProfileDataComparison.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProfileDataComparison.Responses.$200
    | Paths.ListProfileDataComparison.Responses.$400
    | Paths.ListProfileDataComparison.Responses.$401
    | Paths.ListProfileDataComparison.Responses.$404
  >

  'listDataRequirementDataComparison'(
    parameters?:
      | Parameters<Paths.ListDataRequirementDataComparison.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDataRequirementDataComparison.Responses.$200
    | Paths.ListDataRequirementDataComparison.Responses.$400
    | Paths.ListDataRequirementDataComparison.Responses.$401
    | Paths.ListDataRequirementDataComparison.Responses.$404
  >

  'listDataComparisonComparisonResult'(
    parameters?:
      | Parameters<Paths.ListDataComparisonComparisonResult.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDataComparisonComparisonResult.Responses.$200
    | Paths.ListDataComparisonComparisonResult.Responses.$400
    | Paths.ListDataComparisonComparisonResult.Responses.$401
    | Paths.ListDataComparisonComparisonResult.Responses.$404
  >

  'getComparisonResult'(
    parameters?: Parameters<Paths.GetComparisonResult.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetComparisonResult.Responses.$200
    | Paths.GetComparisonResult.Responses.$400
    | Paths.GetComparisonResult.Responses.$401
    | Paths.GetComparisonResult.Responses.$404
  >

  'listDataRequirementEvidenceSubmission'(
    parameters?:
      | Parameters<Paths.ListDataRequirementEvidenceSubmission.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDataRequirementEvidenceSubmission.Responses.$200
    | Paths.ListDataRequirementEvidenceSubmission.Responses.$400
    | Paths.ListDataRequirementEvidenceSubmission.Responses.$401
    | Paths.ListDataRequirementEvidenceSubmission.Responses.$404
  >

  'getEvidenceSubmission'(
    parameters?: Parameters<Paths.GetEvidenceSubmission.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetEvidenceSubmission.Responses.$200
    | Paths.GetEvidenceSubmission.Responses.$400
    | Paths.GetEvidenceSubmission.Responses.$401
    | Paths.GetEvidenceSubmission.Responses.$404
  >

  'updateEvidenceSubmission'(
    parameters?: Parameters<Paths.UpdateEvidenceSubmission.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateEvidenceSubmission.Responses.$200
    | Paths.UpdateEvidenceSubmission.Responses.$400
    | Paths.UpdateEvidenceSubmission.Responses.$401
    | Paths.UpdateEvidenceSubmission.Responses.$404
    | Paths.UpdateEvidenceSubmission.Responses.$406
    | Paths.UpdateEvidenceSubmission.Responses.$409
  >

  'updateEvidenceSubmissionReject'(
    parameters?:
      | Parameters<Paths.UpdateEvidenceSubmissionReject.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateEvidenceSubmissionReject.Responses.$200
    | Paths.UpdateEvidenceSubmissionReject.Responses.$400
    | Paths.UpdateEvidenceSubmissionReject.Responses.$401
    | Paths.UpdateEvidenceSubmissionReject.Responses.$404
    | Paths.UpdateEvidenceSubmissionReject.Responses.$406
  >

  'updateEvidenceSubmissionApprove'(
    parameters?:
      | Parameters<Paths.UpdateEvidenceSubmissionApprove.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateEvidenceSubmissionApprove.Responses.$200
    | Paths.UpdateEvidenceSubmissionApprove.Responses.$400
    | Paths.UpdateEvidenceSubmissionApprove.Responses.$401
    | Paths.UpdateEvidenceSubmissionApprove.Responses.$404
    | Paths.UpdateEvidenceSubmissionApprove.Responses.$406
  >

  'updateEvidenceSubmissionReplace'(
    parameters?:
      | Parameters<Paths.UpdateEvidenceSubmissionReplace.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateEvidenceSubmissionReplace.Responses.$200
    | Paths.UpdateEvidenceSubmissionReplace.Responses.$400
    | Paths.UpdateEvidenceSubmissionReplace.Responses.$401
    | Paths.UpdateEvidenceSubmissionReplace.Responses.$404
    | Paths.UpdateEvidenceSubmissionReplace.Responses.$406
  >

  'listEvidenceSubmissionQualitativeCheck'(
    parameters?:
      | Parameters<Paths.ListEvidenceSubmissionQualitativeCheck.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$200
    | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$400
    | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$401
    | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$404
  >

  'getQualitativeCheck'(
    parameters?: Parameters<Paths.GetQualitativeCheck.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetQualitativeCheck.Responses.$200
    | Paths.GetQualitativeCheck.Responses.$400
    | Paths.GetQualitativeCheck.Responses.$401
    | Paths.GetQualitativeCheck.Responses.$404
  >

  'listEvidenceSubmissionRejectionReason'(
    parameters?:
      | Parameters<Paths.ListEvidenceSubmissionRejectionReason.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListEvidenceSubmissionRejectionReason.Responses.$200
    | Paths.ListEvidenceSubmissionRejectionReason.Responses.$400
    | Paths.ListEvidenceSubmissionRejectionReason.Responses.$401
    | Paths.ListEvidenceSubmissionRejectionReason.Responses.$404
  >

  'getRejectionReason'(
    parameters?: Parameters<Paths.GetRejectionReason.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetRejectionReason.Responses.$200
    | Paths.GetRejectionReason.Responses.$400
    | Paths.GetRejectionReason.Responses.$401
    | Paths.GetRejectionReason.Responses.$404
  >

  'listEvidenceSubmissionDocument'(
    parameters?:
      | Parameters<Paths.ListEvidenceSubmissionDocument.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListEvidenceSubmissionDocument.Responses.$200
    | Paths.ListEvidenceSubmissionDocument.Responses.$400
    | Paths.ListEvidenceSubmissionDocument.Responses.$401
    | Paths.ListEvidenceSubmissionDocument.Responses.$404
  >

  'createEvidenceSubmissionDocument'(
    parameters?:
      | Parameters<Paths.CreateEvidenceSubmissionDocument.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateEvidenceSubmissionDocument.Responses.$201
    | Paths.CreateEvidenceSubmissionDocument.Responses.$400
    | Paths.CreateEvidenceSubmissionDocument.Responses.$404
    | Paths.CreateEvidenceSubmissionDocument.Responses.$422
  >

  'getDocument'(
    parameters?: Parameters<Paths.GetDocument.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDocument.Responses.$200
    | Paths.GetDocument.Responses.$400
    | Paths.GetDocument.Responses.$401
    | Paths.GetDocument.Responses.$404
  >

  'listDocumentDocumentToken'(
    parameters?: Parameters<Paths.ListDocumentDocumentToken.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDocumentDocumentToken.Responses.$200
    | Paths.ListDocumentDocumentToken.Responses.$400
    | Paths.ListDocumentDocumentToken.Responses.$401
    | Paths.ListDocumentDocumentToken.Responses.$404
  >

  'createDocumentDocumentToken'(
    parameters?:
      | Parameters<Paths.CreateDocumentDocumentToken.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateDocumentDocumentToken.Responses.$201
    | Paths.CreateDocumentDocumentToken.Responses.$400
    | Paths.CreateDocumentDocumentToken.Responses.$401
    | Paths.CreateDocumentDocumentToken.Responses.$404
    | Paths.CreateDocumentDocumentToken.Responses.$406
  >

  'getDocumentToken'(
    parameters?: Parameters<Paths.GetDocumentToken.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDocumentToken.Responses.$200
    | Paths.GetDocumentToken.Responses.$400
    | Paths.GetDocumentToken.Responses.$401
    | Paths.GetDocumentToken.Responses.$404
  >

  'getDocumentFileDownload'(
    parameters?: Parameters<Paths.GetDocumentFileDownload.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDocumentFileDownload.Responses.$302
    | Paths.GetDocumentFileDownload.Responses.$404
    | Paths.GetDocumentFileDownload.Responses.$406
    | Paths.GetDocumentFileDownload.Responses.$410
  >

  'getForm'(
    parameters?: Parameters<Paths.GetForm.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetForm.Responses.$200
    | Paths.GetForm.Responses.$400
    | Paths.GetForm.Responses.$401
    | Paths.GetForm.Responses.$404
  >

  'getFormFormData'(
    parameters?: Parameters<Paths.GetFormFormData.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetFormFormData.Responses.$200
    | Paths.GetFormFormData.Responses.$400
    | Paths.GetFormFormData.Responses.$401
    | Paths.GetFormFormData.Responses.$404
  >

  'createForm'(
    parameters?: Parameters<Paths.CreateForm.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateForm.Responses.$201
    | Paths.CreateForm.Responses.$404
    | Paths.CreateForm.Responses.$422
  >

  'getFormDataById'(
    parameters?: Parameters<Paths.GetFormDataById.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetFormDataById.Responses.$200
    | Paths.GetFormDataById.Responses.$400
    | Paths.GetFormDataById.Responses.$401
    | Paths.GetFormDataById.Responses.$404
  >

  'listKycApplicationRelatedEntitie'(
    parameters?:
      | Parameters<Paths.ListKycApplicationRelatedEntitie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationRelatedEntitie.Responses.$200
    | Paths.ListKycApplicationRelatedEntitie.Responses.$400
    | Paths.ListKycApplicationRelatedEntitie.Responses.$401
    | Paths.ListKycApplicationRelatedEntitie.Responses.$404
    | Paths.ListKycApplicationRelatedEntitie.Responses.$409
  >

  'createKycApplicationRelatedEntitie'(
    parameters?:
      | Parameters<Paths.CreateKycApplicationRelatedEntitie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$201
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$400
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$401
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$404
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$406
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$409
    | Paths.CreateKycApplicationRelatedEntitie.Responses.$422
  >

  'getRelatedEntitie'(
    parameters?: Parameters<Paths.GetRelatedEntitie.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetRelatedEntitie.Responses.$200
    | Paths.GetRelatedEntitie.Responses.$400
    | Paths.GetRelatedEntitie.Responses.$401
    | Paths.GetRelatedEntitie.Responses.$404
  >

  'deleteRelatedEntitie'(
    parameters?: Parameters<Paths.DeleteRelatedEntitie.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteRelatedEntitie.Responses.$204
    | Paths.DeleteRelatedEntitie.Responses.$400
    | Paths.DeleteRelatedEntitie.Responses.$401
    | Paths.DeleteRelatedEntitie.Responses.$404
    | Paths.DeleteRelatedEntitie.Responses.$422
  >

  'listKycApplicationPdfReportSummarie'(
    parameters?:
      | Parameters<Paths.ListKycApplicationPdfReportSummarie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationPdfReportSummarie.Responses.$200
    | Paths.ListKycApplicationPdfReportSummarie.Responses.$400
    | Paths.ListKycApplicationPdfReportSummarie.Responses.$401
    | Paths.ListKycApplicationPdfReportSummarie.Responses.$404
  >

  'createKycApplicationPdfReportSummarie'(
    parameters?:
      | Parameters<Paths.CreateKycApplicationPdfReportSummarie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$201
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$400
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$401
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$404
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$406
    | Paths.CreateKycApplicationPdfReportSummarie.Responses.$429
  >

  'getPdfReportSummarie'(
    parameters?: Parameters<Paths.GetPdfReportSummarie.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetPdfReportSummarie.Responses.$200
    | Paths.GetPdfReportSummarie.Responses.$400
    | Paths.GetPdfReportSummarie.Responses.$401
    | Paths.GetPdfReportSummarie.Responses.$404
  >

  'getPdfReportSummarieDownload'(
    parameters?:
      | Parameters<Paths.GetPdfReportSummarieDownload.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetPdfReportSummarieDownload.Responses.$302
    | Paths.GetPdfReportSummarieDownload.Responses.$400
    | Paths.GetPdfReportSummarieDownload.Responses.$401
    | Paths.GetPdfReportSummarieDownload.Responses.$404
    | Paths.GetPdfReportSummarieDownload.Responses.$406
    | Paths.GetPdfReportSummarieDownload.Responses.$409
  >

  'listUser'(
    parameters?: Parameters<Paths.ListUser.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListUser.Responses.$200
    | Paths.ListUser.Responses.$400
    | Paths.ListUser.Responses.$401
    | Paths.ListUser.Responses.$403
  >

  'getUser'(
    parameters?: Parameters<Paths.GetUser.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetUser.Responses.$200
    | Paths.GetUser.Responses.$400
    | Paths.GetUser.Responses.$401
    | Paths.GetUser.Responses.$404
  >

  'getRiskRatingEvaluation'(
    parameters?: Parameters<Paths.GetRiskRatingEvaluation.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetRiskRatingEvaluation.Responses.$200
    | Paths.GetRiskRatingEvaluation.Responses.$400
    | Paths.GetRiskRatingEvaluation.Responses.$401
    | Paths.GetRiskRatingEvaluation.Responses.$404
  >

  'listKycApplicationRiskRatingEvaluation'(
    parameters?:
      | Parameters<Paths.ListKycApplicationRiskRatingEvaluation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$200
    | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$400
    | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$401
    | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$404
  >

  'listKycApplicationRiskRating'(
    parameters?:
      | Parameters<Paths.ListKycApplicationRiskRating.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationRiskRating.Responses.$200
    | Paths.ListKycApplicationRiskRating.Responses.$400
    | Paths.ListKycApplicationRiskRating.Responses.$401
    | Paths.ListKycApplicationRiskRating.Responses.$404
  >

  'createKycApplicationRiskRating'(
    parameters?:
      | Parameters<Paths.CreateKycApplicationRiskRating.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplicationRiskRating.Responses.$201
    | Paths.CreateKycApplicationRiskRating.Responses.$400
    | Paths.CreateKycApplicationRiskRating.Responses.$401
    | Paths.CreateKycApplicationRiskRating.Responses.$404
    | Paths.CreateKycApplicationRiskRating.Responses.$406
  >

  'getKycApplicationRiskRating'(
    parameters?:
      | Parameters<Paths.GetKycApplicationRiskRating.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetKycApplicationRiskRating.Responses.$200
    | Paths.GetKycApplicationRiskRating.Responses.$400
    | Paths.GetKycApplicationRiskRating.Responses.$401
    | Paths.GetKycApplicationRiskRating.Responses.$404
  >

  'updateKycApplicationRiskRating'(
    parameters?:
      | Parameters<Paths.UpdateKycApplicationRiskRating.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateKycApplicationRiskRating.Responses.$200
    | Paths.UpdateKycApplicationRiskRating.Responses.$400
    | Paths.UpdateKycApplicationRiskRating.Responses.$401
    | Paths.UpdateKycApplicationRiskRating.Responses.$404
    | Paths.UpdateKycApplicationRiskRating.Responses.$406
  >

  'listKycApplicationRiskRatingsQuestion'(
    parameters?:
      | Parameters<Paths.ListKycApplicationRiskRatingsQuestion.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationRiskRatingsQuestion.Responses.$200
    | Paths.ListKycApplicationRiskRatingsQuestion.Responses.$404
  >

  'getBankAccountSchema'(
    parameters?: Parameters<Paths.GetBankAccountSchema.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetBankAccountSchema.Responses.$200
    | Paths.GetBankAccountSchema.Responses.$400
    | Paths.GetBankAccountSchema.Responses.$401
    | Paths.GetBankAccountSchema.Responses.$404
  >

  'listKycApplicationBankAccount'(
    parameters?:
      | Parameters<Paths.ListKycApplicationBankAccount.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationBankAccount.Responses.$200
    | Paths.ListKycApplicationBankAccount.Responses.$400
    | Paths.ListKycApplicationBankAccount.Responses.$401
    | Paths.ListKycApplicationBankAccount.Responses.$404
  >

  'createKycApplicationBankAccount'(
    parameters?:
      | Parameters<Paths.CreateKycApplicationBankAccount.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplicationBankAccount.Responses.$201
    | Paths.CreateKycApplicationBankAccount.Responses.$400
    | Paths.CreateKycApplicationBankAccount.Responses.$401
    | Paths.CreateKycApplicationBankAccount.Responses.$404
    | Paths.CreateKycApplicationBankAccount.Responses.$406
    | Paths.CreateKycApplicationBankAccount.Responses.$422
  >

  'getBankAccount'(
    parameters?: Parameters<Paths.GetBankAccount.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetBankAccount.Responses.$200
    | Paths.GetBankAccount.Responses.$400
    | Paths.GetBankAccount.Responses.$401
    | Paths.GetBankAccount.Responses.$404
  >

  'getBankAccountVerification'(
    parameters?:
      | Parameters<Paths.GetBankAccountVerification.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetBankAccountVerification.Responses.$200
    | Paths.GetBankAccountVerification.Responses.$400
    | Paths.GetBankAccountVerification.Responses.$401
    | Paths.GetBankAccountVerification.Responses.$404
  >

  'updateBankAccountVerification'(
    parameters?:
      | Parameters<Paths.UpdateBankAccountVerification.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateBankAccountVerification.Responses.$200
    | Paths.UpdateBankAccountVerification.Responses.$400
    | Paths.UpdateBankAccountVerification.Responses.$401
    | Paths.UpdateBankAccountVerification.Responses.$404
    | Paths.UpdateBankAccountVerification.Responses.$406
  >

  'listBankAccountBankAccountVerification'(
    parameters?:
      | Parameters<Paths.ListBankAccountBankAccountVerification.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListBankAccountBankAccountVerification.Responses.$200
    | Paths.ListBankAccountBankAccountVerification.Responses.$400
    | Paths.ListBankAccountBankAccountVerification.Responses.$401
    | Paths.ListBankAccountBankAccountVerification.Responses.$404
  >

  'listKycApplicationConsumerReportQuerie'(
    parameters?:
      | Parameters<Paths.ListKycApplicationConsumerReportQuerie.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationConsumerReportQuerie.Responses.$200
    | Paths.ListKycApplicationConsumerReportQuerie.Responses.$400
    | Paths.ListKycApplicationConsumerReportQuerie.Responses.$401
    | Paths.ListKycApplicationConsumerReportQuerie.Responses.$404
  >

  'getConsumerReportQuerie'(
    parameters?: Parameters<Paths.GetConsumerReportQuerie.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetConsumerReportQuerie.Responses.$200
    | Paths.GetConsumerReportQuerie.Responses.$400
    | Paths.GetConsumerReportQuerie.Responses.$401
    | Paths.GetConsumerReportQuerie.Responses.$404
  >

  'getConsumerReportQuerieDownload'(
    parameters?:
      | Parameters<Paths.GetConsumerReportQuerieDownload.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetConsumerReportQuerieDownload.Responses.$302
    | Paths.GetConsumerReportQuerieDownload.Responses.$400
    | Paths.GetConsumerReportQuerieDownload.Responses.$401
    | Paths.GetConsumerReportQuerieDownload.Responses.$404
    | Paths.GetConsumerReportQuerieDownload.Responses.$406
    | Paths.GetConsumerReportQuerieDownload.Responses.$409
  >

  'createOnboardingInvitation'(
    parameters?:
      | Parameters<Paths.CreateOnboardingInvitation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateOnboardingInvitation.Responses.$201
    | Paths.CreateOnboardingInvitation.Responses.$400
    | Paths.CreateOnboardingInvitation.Responses.$401
    | Paths.CreateOnboardingInvitation.Responses.$403
    | Paths.CreateOnboardingInvitation.Responses.$406
  >

  'updateOnboardingInvitation'(
    parameters?:
      | Parameters<Paths.UpdateOnboardingInvitation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateOnboardingInvitation.Responses.$200
    | Paths.UpdateOnboardingInvitation.Responses.$400
    | Paths.UpdateOnboardingInvitation.Responses.$401
    | Paths.UpdateOnboardingInvitation.Responses.$404
    | Paths.UpdateOnboardingInvitation.Responses.$406
    | Paths.UpdateOnboardingInvitation.Responses.$422
  >

  'listFormFormPdfExport'(
    parameters?: Parameters<Paths.ListFormFormPdfExport.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListFormFormPdfExport.Responses.$200
    | Paths.ListFormFormPdfExport.Responses.$400
    | Paths.ListFormFormPdfExport.Responses.$401
    | Paths.ListFormFormPdfExport.Responses.$404
  >

  'createFormFormPdfExport'(
    parameters?: Parameters<Paths.CreateFormFormPdfExport.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateFormFormPdfExport.Responses.$201
    | Paths.CreateFormFormPdfExport.Responses.$400
    | Paths.CreateFormFormPdfExport.Responses.$401
    | Paths.CreateFormFormPdfExport.Responses.$404
    | Paths.CreateFormFormPdfExport.Responses.$406
    | Paths.CreateFormFormPdfExport.Responses.$429
  >

  'getFormPdfExport'(
    parameters?: Parameters<Paths.GetFormPdfExport.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetFormPdfExport.Responses.$200
    | Paths.GetFormPdfExport.Responses.$400
    | Paths.GetFormPdfExport.Responses.$401
    | Paths.GetFormPdfExport.Responses.$404
  >

  'getFormPdfExportDownload'(
    parameters?: Parameters<Paths.GetFormPdfExportDownload.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetFormPdfExportDownload.Responses.$302
    | Paths.GetFormPdfExportDownload.Responses.$400
    | Paths.GetFormPdfExportDownload.Responses.$401
    | Paths.GetFormPdfExportDownload.Responses.$404
    | Paths.GetFormPdfExportDownload.Responses.$406
    | Paths.GetFormPdfExportDownload.Responses.$409
  >

  'listKycEntityTemplateDesignationTemplate'(
    parameters?:
      | Parameters<Paths.ListKycEntityTemplateDesignationTemplate.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$200
    | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$400
    | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$401
    | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$404
  >

  'getRelatedEntityDesignation'(
    parameters?:
      | Parameters<Paths.GetRelatedEntityDesignation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetRelatedEntityDesignation.Responses.$200
    | Paths.GetRelatedEntityDesignation.Responses.$400
    | Paths.GetRelatedEntityDesignation.Responses.$401
    | Paths.GetRelatedEntityDesignation.Responses.$404
  >

  'deleteRelatedEntityDesignation'(
    parameters?:
      | Parameters<Paths.DeleteRelatedEntityDesignation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.DeleteRelatedEntityDesignation.Responses.$204
    | Paths.DeleteRelatedEntityDesignation.Responses.$400
    | Paths.DeleteRelatedEntityDesignation.Responses.$401
    | Paths.DeleteRelatedEntityDesignation.Responses.$404
  >

  'listRelatedEntitieRelatedEntityDesignation'(
    parameters?:
      | Parameters<Paths.ListRelatedEntitieRelatedEntityDesignation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$200
    | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$400
    | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$401
    | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$404
  >

  'createRelatedEntitieRelatedEntityDesignation'(
    parameters?:
      | Parameters<Paths.CreateRelatedEntitieRelatedEntityDesignation.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$201
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$400
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$401
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$404
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$406
    | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$422
  >

  'listProductProductProductAddon'(
    parameters?:
      | Parameters<Paths.ListProductProductProductAddon.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProductProductProductAddon.Responses.$200
    | Paths.ListProductProductProductAddon.Responses.$400
    | Paths.ListProductProductProductAddon.Responses.$401
    | Paths.ListProductProductProductAddon.Responses.$404
  >

  'listAccountApplicationAccountAddon'(
    parameters?:
      | Parameters<Paths.ListAccountApplicationAccountAddon.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListAccountApplicationAccountAddon.Responses.$200
    | Paths.ListAccountApplicationAccountAddon.Responses.$400
    | Paths.ListAccountApplicationAccountAddon.Responses.$401
    | Paths.ListAccountApplicationAccountAddon.Responses.$404
  >

  'getAccountAddon'(
    parameters?: Parameters<Paths.GetAccountAddon.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAccountAddon.Responses.$200
    | Paths.GetAccountAddon.Responses.$400
    | Paths.GetAccountAddon.Responses.$401
    | Paths.GetAccountAddon.Responses.$404
  >

  'listProductDisclosure'(
    parameters?: Parameters<Paths.ListProductDisclosure.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProductDisclosure.Responses.$200
    | Paths.ListProductDisclosure.Responses.$400
    | Paths.ListProductDisclosure.Responses.$401
    | Paths.ListProductDisclosure.Responses.$403
    | Paths.ListProductDisclosure.Responses.$404
  >

  'listDisclosure'(
    parameters?: Parameters<Paths.ListDisclosure.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListDisclosure.Responses.$200
    | Paths.ListDisclosure.Responses.$400
    | Paths.ListDisclosure.Responses.$401
    | Paths.ListDisclosure.Responses.$403
  >

  'getDisclosure'(
    parameters?: Parameters<Paths.GetDisclosure.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetDisclosure.Responses.$200
    | Paths.GetDisclosure.Responses.$400
    | Paths.GetDisclosure.Responses.$401
    | Paths.GetDisclosure.Responses.$404
  >

  'listProduct'(
    parameters?: Parameters<Paths.ListProduct.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProduct.Responses.$200
    | Paths.ListProduct.Responses.$400
    | Paths.ListProduct.Responses.$401
    | Paths.ListProduct.Responses.$403
  >

  'getProduct'(
    parameters?: Parameters<Paths.GetProduct.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetProduct.Responses.$200
    | Paths.GetProduct.Responses.$400
    | Paths.GetProduct.Responses.$401
    | Paths.GetProduct.Responses.$404
  >

  'listKycEntityTemplateProduct'(
    parameters?:
      | Parameters<Paths.ListKycEntityTemplateProduct.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycEntityTemplateProduct.Responses.$200
    | Paths.ListKycEntityTemplateProduct.Responses.$400
    | Paths.ListKycEntityTemplateProduct.Responses.$401
    | Paths.ListKycEntityTemplateProduct.Responses.$404
  >

  'getAccountApplicationAccountSummaryPdf'(
    parameters?:
      | Parameters<Paths.GetAccountApplicationAccountSummaryPdf.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAccountApplicationAccountSummaryPdf.Responses.$200
    | Paths.GetAccountApplicationAccountSummaryPdf.Responses.$404
  >

  'listKycEntityTemplateDisclosure'(
    parameters?:
      | Parameters<Paths.ListKycEntityTemplateDisclosure.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycEntityTemplateDisclosure.Responses.$200
    | Paths.ListKycEntityTemplateDisclosure.Responses.$400
    | Paths.ListKycEntityTemplateDisclosure.Responses.$401
    | Paths.ListKycEntityTemplateDisclosure.Responses.$403
    | Paths.ListKycEntityTemplateDisclosure.Responses.$404
  >

  'listAccountApplicationDataRequirement'(
    parameters?:
      | Parameters<Paths.ListAccountApplicationDataRequirement.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListAccountApplicationDataRequirement.Responses.$200
    | Paths.ListAccountApplicationDataRequirement.Responses.$400
    | Paths.ListAccountApplicationDataRequirement.Responses.$401
    | Paths.ListAccountApplicationDataRequirement.Responses.$404
  >

  'listProductProductAddonsDisclosure'(
    parameters?:
      | Parameters<Paths.ListProductProductAddonsDisclosure.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListProductProductAddonsDisclosure.Responses.$200
    | Paths.ListProductProductAddonsDisclosure.Responses.$400
    | Paths.ListProductProductAddonsDisclosure.Responses.$401
    | Paths.ListProductProductAddonsDisclosure.Responses.$403
  >

  'getAccountApplicationAccountSummary'(
    parameters?:
      | Parameters<Paths.GetAccountApplicationAccountSummary.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAccountApplicationAccountSummary.Responses.$200
    | Paths.GetAccountApplicationAccountSummary.Responses.$404
  >

  'listKycApplicationAccountApplication'(
    parameters?:
      | Parameters<Paths.ListKycApplicationAccountApplication.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.ListKycApplicationAccountApplication.Responses.$200
    | Paths.ListKycApplicationAccountApplication.Responses.$400
    | Paths.ListKycApplicationAccountApplication.Responses.$401
    | Paths.ListKycApplicationAccountApplication.Responses.$404
  >

  'createKycApplicationAccountApplication'(
    parameters?:
      | Parameters<Paths.CreateKycApplicationAccountApplication.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.CreateKycApplicationAccountApplication.Responses.$201
    | Paths.CreateKycApplicationAccountApplication.Responses.$400
    | Paths.CreateKycApplicationAccountApplication.Responses.$401
    | Paths.CreateKycApplicationAccountApplication.Responses.$404
    | Paths.CreateKycApplicationAccountApplication.Responses.$406
  >

  'getAccountApplication'(
    parameters?: Parameters<Paths.GetAccountApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.GetAccountApplication.Responses.$200
    | Paths.GetAccountApplication.Responses.$400
    | Paths.GetAccountApplication.Responses.$401
    | Paths.GetAccountApplication.Responses.$404
  >

  'updateAccountApplication'(
    parameters?: Parameters<Paths.UpdateAccountApplication.OperationParameters> | null | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateAccountApplication.Responses.$200
    | Paths.UpdateAccountApplication.Responses.$400
    | Paths.UpdateAccountApplication.Responses.$401
    | Paths.UpdateAccountApplication.Responses.$404
    | Paths.UpdateAccountApplication.Responses.$406
  >

  'updateAccountApplicationDisclosureAcceptances'(
    parameters?:
      | Parameters<Paths.UpdateAccountApplicationDisclosureAcceptances.OperationParameters>
      | null
      | undefined,
    payload?: any,
    config?: AxiosRequestConfig
  ): OperationResponse<
    | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$200
    | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$400
    | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$401
    | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$404
    | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$406
  >
}

export interface OperationMethodResponses {
  requestAuthentication: {
    $200: Paths.RequestAuthentication.Responses.$200
    $400: Paths.RequestAuthentication.Responses.$400
    $401: Paths.RequestAuthentication.Responses.$401
  }

  requestToken: {
    $200: Paths.RequestToken.Responses.$200
    $400: Paths.RequestToken.Responses.$400
    $401: Paths.RequestToken.Responses.$401
  }

  listKycApplication: {
    $200: Paths.ListKycApplication.Responses.$200
    $400: Paths.ListKycApplication.Responses.$400
    $401: Paths.ListKycApplication.Responses.$401
    $403: Paths.ListKycApplication.Responses.$403
  }

  createKycApplication: {
    $201: Paths.CreateKycApplication.Responses.$201
    $400: Paths.CreateKycApplication.Responses.$400
    $401: Paths.CreateKycApplication.Responses.$401
    $406: Paths.CreateKycApplication.Responses.$406
    $422: Paths.CreateKycApplication.Responses.$422
  }

  getKycApplication: {
    $200: Paths.GetKycApplication.Responses.$200
    $400: Paths.GetKycApplication.Responses.$400
    $401: Paths.GetKycApplication.Responses.$401
    $404: Paths.GetKycApplication.Responses.$404
  }

  deleteKycApplication: {
    $204: Paths.DeleteKycApplication.Responses.$204
    $400: Paths.DeleteKycApplication.Responses.$400
    $401: Paths.DeleteKycApplication.Responses.$401
    $404: Paths.DeleteKycApplication.Responses.$404
  }

  updateKycApplicationApprove: {
    $200: Paths.UpdateKycApplicationApprove.Responses.$200
    $400: Paths.UpdateKycApplicationApprove.Responses.$400
    $401: Paths.UpdateKycApplicationApprove.Responses.$401
    $404: Paths.UpdateKycApplicationApprove.Responses.$404
    $406: Paths.UpdateKycApplicationApprove.Responses.$406
    $422: Paths.UpdateKycApplicationApprove.Responses.$422
  }

  updateKycApplicationUnapprove: {
    $200: Paths.UpdateKycApplicationUnapprove.Responses.$200
    $400: Paths.UpdateKycApplicationUnapprove.Responses.$400
    $401: Paths.UpdateKycApplicationUnapprove.Responses.$401
    $404: Paths.UpdateKycApplicationUnapprove.Responses.$404
    $406: Paths.UpdateKycApplicationUnapprove.Responses.$406
    $422: Paths.UpdateKycApplicationUnapprove.Responses.$422
  }

  updateKycApplicationTransfer: {
    $200: Paths.UpdateKycApplicationTransfer.Responses.$200
    $400: Paths.UpdateKycApplicationTransfer.Responses.$400
    $401: Paths.UpdateKycApplicationTransfer.Responses.$401
    $404: Paths.UpdateKycApplicationTransfer.Responses.$404
    $406: Paths.UpdateKycApplicationTransfer.Responses.$406
    $422: Paths.UpdateKycApplicationTransfer.Responses.$422
  }

  archiveKycApplication: {
    $204: Paths.ArchiveKycApplication.Responses.$204
    $400: Paths.ArchiveKycApplication.Responses.$400
  }

  unarchiveKycApplication: {
    $200: Paths.UnarchiveKycApplication.Responses.$200
    $400: Paths.UnarchiveKycApplication.Responses.$400
  }

  listKycApplicationStatusSummarie: {
    $200: Paths.ListKycApplicationStatusSummarie.Responses.$200
    $400: Paths.ListKycApplicationStatusSummarie.Responses.$400
    $401: Paths.ListKycApplicationStatusSummarie.Responses.$401
    $404: Paths.ListKycApplicationStatusSummarie.Responses.$404
  }

  listKycApplicationProfile: {
    $200: Paths.ListKycApplicationProfile.Responses.$200
    $400: Paths.ListKycApplicationProfile.Responses.$400
    $401: Paths.ListKycApplicationProfile.Responses.$401
    $404: Paths.ListKycApplicationProfile.Responses.$404
  }

  getProfile: {
    $200: Paths.GetProfile.Responses.$200
    $400: Paths.GetProfile.Responses.$400
    $401: Paths.GetProfile.Responses.$401
    $404: Paths.GetProfile.Responses.$404
  }

  listProfileName: {
    $200: Paths.ListProfileName.Responses.$200
    $400: Paths.ListProfileName.Responses.$400
    $401: Paths.ListProfileName.Responses.$401
    $404: Paths.ListProfileName.Responses.$404
  }

  createProfileName: {
    $201: Paths.CreateProfileName.Responses.$201
    $400: Paths.CreateProfileName.Responses.$400
    $401: Paths.CreateProfileName.Responses.$401
    $404: Paths.CreateProfileName.Responses.$404
    $406: Paths.CreateProfileName.Responses.$406
    $422: Paths.CreateProfileName.Responses.$422
  }

  getName: {
    $200: Paths.GetName.Responses.$200
    $400: Paths.GetName.Responses.$400
    $401: Paths.GetName.Responses.$401
    $404: Paths.GetName.Responses.$404
  }

  deleteName: {
    $204: Paths.DeleteName.Responses.$204
    $400: Paths.DeleteName.Responses.$400
    $401: Paths.DeleteName.Responses.$401
    $404: Paths.DeleteName.Responses.$404
  }

  updateName: {
    $200: Paths.UpdateName.Responses.$200
    $400: Paths.UpdateName.Responses.$400
    $401: Paths.UpdateName.Responses.$401
    $404: Paths.UpdateName.Responses.$404
    $406: Paths.UpdateName.Responses.$406
    $422: Paths.UpdateName.Responses.$422
  }

  listProfileNumber: {
    $200: Paths.ListProfileNumber.Responses.$200
    $400: Paths.ListProfileNumber.Responses.$400
    $401: Paths.ListProfileNumber.Responses.$401
    $404: Paths.ListProfileNumber.Responses.$404
  }

  createProfileNumber: {
    $201: Paths.CreateProfileNumber.Responses.$201
    $400: Paths.CreateProfileNumber.Responses.$400
    $401: Paths.CreateProfileNumber.Responses.$401
    $404: Paths.CreateProfileNumber.Responses.$404
    $406: Paths.CreateProfileNumber.Responses.$406
    $422: Paths.CreateProfileNumber.Responses.$422
  }

  getNumber: {
    $200: Paths.GetNumber.Responses.$200
    $400: Paths.GetNumber.Responses.$400
    $401: Paths.GetNumber.Responses.$401
    $404: Paths.GetNumber.Responses.$404
  }

  deleteNumber: {
    $204: Paths.DeleteNumber.Responses.$204
    $400: Paths.DeleteNumber.Responses.$400
    $401: Paths.DeleteNumber.Responses.$401
    $404: Paths.DeleteNumber.Responses.$404
    $422: Paths.DeleteNumber.Responses.$422
  }

  updateNumber: {
    $200: Paths.UpdateNumber.Responses.$200
    $400: Paths.UpdateNumber.Responses.$400
    $401: Paths.UpdateNumber.Responses.$401
    $404: Paths.UpdateNumber.Responses.$404
    $406: Paths.UpdateNumber.Responses.$406
    $422: Paths.UpdateNumber.Responses.$422
  }

  listProfileContact: {
    $200: Paths.ListProfileContact.Responses.$200
    $400: Paths.ListProfileContact.Responses.$400
    $401: Paths.ListProfileContact.Responses.$401
    $404: Paths.ListProfileContact.Responses.$404
  }

  createProfileContact: {
    $201: Paths.CreateProfileContact.Responses.$201
    $400: Paths.CreateProfileContact.Responses.$400
    $401: Paths.CreateProfileContact.Responses.$401
    $404: Paths.CreateProfileContact.Responses.$404
    $406: Paths.CreateProfileContact.Responses.$406
    $422: Paths.CreateProfileContact.Responses.$422
  }

  getContact: {
    $200: Paths.GetContact.Responses.$200
    $400: Paths.GetContact.Responses.$400
    $401: Paths.GetContact.Responses.$401
    $404: Paths.GetContact.Responses.$404
  }

  deleteContact: {
    $204: Paths.DeleteContact.Responses.$204
    $400: Paths.DeleteContact.Responses.$400
    $401: Paths.DeleteContact.Responses.$401
    $404: Paths.DeleteContact.Responses.$404
  }

  updateContact: {
    $200: Paths.UpdateContact.Responses.$200
    $400: Paths.UpdateContact.Responses.$400
    $401: Paths.UpdateContact.Responses.$401
    $404: Paths.UpdateContact.Responses.$404
    $406: Paths.UpdateContact.Responses.$406
    $422: Paths.UpdateContact.Responses.$422
  }

  listProfileAddresse: {
    $200: Paths.ListProfileAddresse.Responses.$200
    $400: Paths.ListProfileAddresse.Responses.$400
    $401: Paths.ListProfileAddresse.Responses.$401
    $404: Paths.ListProfileAddresse.Responses.$404
  }

  createProfileAddresse: {
    $201: Paths.CreateProfileAddresse.Responses.$201
    $400: Paths.CreateProfileAddresse.Responses.$400
    $401: Paths.CreateProfileAddresse.Responses.$401
    $404: Paths.CreateProfileAddresse.Responses.$404
    $406: Paths.CreateProfileAddresse.Responses.$406
    $422: Paths.CreateProfileAddresse.Responses.$422
  }

  getAddresse: {
    $200: Paths.GetAddresse.Responses.$200
    $400: Paths.GetAddresse.Responses.$400
    $401: Paths.GetAddresse.Responses.$401
    $404: Paths.GetAddresse.Responses.$404
  }

  deleteAddresse: {
    $204: Paths.DeleteAddresse.Responses.$204
    $400: Paths.DeleteAddresse.Responses.$400
    $401: Paths.DeleteAddresse.Responses.$401
    $404: Paths.DeleteAddresse.Responses.$404
  }

  updateAddresse: {
    $200: Paths.UpdateAddresse.Responses.$200
    $400: Paths.UpdateAddresse.Responses.$400
    $401: Paths.UpdateAddresse.Responses.$401
    $404: Paths.UpdateAddresse.Responses.$404
    $406: Paths.UpdateAddresse.Responses.$406
    $422: Paths.UpdateAddresse.Responses.$422
  }

  listProfileAdditionalDetail: {
    $200: Paths.ListProfileAdditionalDetail.Responses.$200
    $400: Paths.ListProfileAdditionalDetail.Responses.$400
    $401: Paths.ListProfileAdditionalDetail.Responses.$401
    $404: Paths.ListProfileAdditionalDetail.Responses.$404
  }

  createProfileAdditionalDetail: {
    $201: Paths.CreateProfileAdditionalDetail.Responses.$201
    $400: Paths.CreateProfileAdditionalDetail.Responses.$400
    $401: Paths.CreateProfileAdditionalDetail.Responses.$401
    $404: Paths.CreateProfileAdditionalDetail.Responses.$404
    $406: Paths.CreateProfileAdditionalDetail.Responses.$406
    $422: Paths.CreateProfileAdditionalDetail.Responses.$422
  }

  getAdditionalDetail: {
    $200: Paths.GetAdditionalDetail.Responses.$200
    $400: Paths.GetAdditionalDetail.Responses.$400
    $401: Paths.GetAdditionalDetail.Responses.$401
    $404: Paths.GetAdditionalDetail.Responses.$404
  }

  deleteAdditionalDetail: {
    $204: Paths.DeleteAdditionalDetail.Responses.$204
    $400: Paths.DeleteAdditionalDetail.Responses.$400
    $401: Paths.DeleteAdditionalDetail.Responses.$401
    $404: Paths.DeleteAdditionalDetail.Responses.$404
    $422: Paths.DeleteAdditionalDetail.Responses.$422
  }

  updateAdditionalDetail: {
    $200: Paths.UpdateAdditionalDetail.Responses.$200
    $400: Paths.UpdateAdditionalDetail.Responses.$400
    $401: Paths.UpdateAdditionalDetail.Responses.$401
    $404: Paths.UpdateAdditionalDetail.Responses.$404
    $406: Paths.UpdateAdditionalDetail.Responses.$406
    $422: Paths.UpdateAdditionalDetail.Responses.$422
  }

  listContactInvitationToken: {
    $200: Paths.ListContactInvitationToken.Responses.$200
    $400: Paths.ListContactInvitationToken.Responses.$400
    $401: Paths.ListContactInvitationToken.Responses.$401
    $404: Paths.ListContactInvitationToken.Responses.$404
  }

  createContactInvitationToken: {
    $201: Paths.CreateContactInvitationToken.Responses.$201
    $400: Paths.CreateContactInvitationToken.Responses.$400
    $401: Paths.CreateContactInvitationToken.Responses.$401
    $404: Paths.CreateContactInvitationToken.Responses.$404
    $406: Paths.CreateContactInvitationToken.Responses.$406
    $409: Paths.CreateContactInvitationToken.Responses.$409
    $422: Paths.CreateContactInvitationToken.Responses.$422
  }

  getInvitationToken: {
    $200: Paths.GetInvitationToken.Responses.$200
    $400: Paths.GetInvitationToken.Responses.$400
    $401: Paths.GetInvitationToken.Responses.$401
    $404: Paths.GetInvitationToken.Responses.$404
  }

  updateInvitationToken: {
    $200: Paths.UpdateInvitationToken.Responses.$200
    $400: Paths.UpdateInvitationToken.Responses.$400
    $401: Paths.UpdateInvitationToken.Responses.$401
    $404: Paths.UpdateInvitationToken.Responses.$404
    $406: Paths.UpdateInvitationToken.Responses.$406
    $409: Paths.UpdateInvitationToken.Responses.$409
  }

  listContactDataRequirementPermission: {
    $200: Paths.ListContactDataRequirementPermission.Responses.$200
    $400: Paths.ListContactDataRequirementPermission.Responses.$400
    $401: Paths.ListContactDataRequirementPermission.Responses.$401
    $404: Paths.ListContactDataRequirementPermission.Responses.$404
  }

  getDataRequirementPermission: {
    $200: Paths.GetDataRequirementPermission.Responses.$200
    $400: Paths.GetDataRequirementPermission.Responses.$400
    $401: Paths.GetDataRequirementPermission.Responses.$401
    $404: Paths.GetDataRequirementPermission.Responses.$404
  }

  deleteDataRequirementPermission: {
    $204: Paths.DeleteDataRequirementPermission.Responses.$204
    $400: Paths.DeleteDataRequirementPermission.Responses.$400
    $401: Paths.DeleteDataRequirementPermission.Responses.$401
    $404: Paths.DeleteDataRequirementPermission.Responses.$404
  }

  createDataRequirementPermission: {
    $201: Paths.CreateDataRequirementPermission.Responses.$201
    $400: Paths.CreateDataRequirementPermission.Responses.$400
    $401: Paths.CreateDataRequirementPermission.Responses.$401
    $404: Paths.CreateDataRequirementPermission.Responses.$404
    $406: Paths.CreateDataRequirementPermission.Responses.$406
    $409: Paths.CreateDataRequirementPermission.Responses.$409
    $422: Paths.CreateDataRequirementPermission.Responses.$422
  }

  listDataRequirementDataRequirementPermission: {
    $200: Paths.ListDataRequirementDataRequirementPermission.Responses.$200
    $400: Paths.ListDataRequirementDataRequirementPermission.Responses.$400
    $401: Paths.ListDataRequirementDataRequirementPermission.Responses.$401
    $404: Paths.ListDataRequirementDataRequirementPermission.Responses.$404
  }

  createDataRequirementDataRequirementPermission: {
    $201: Paths.CreateDataRequirementDataRequirementPermission.Responses.$201
    $400: Paths.CreateDataRequirementDataRequirementPermission.Responses.$400
    $401: Paths.CreateDataRequirementDataRequirementPermission.Responses.$401
    $404: Paths.CreateDataRequirementDataRequirementPermission.Responses.$404
    $406: Paths.CreateDataRequirementDataRequirementPermission.Responses.$406
    $409: Paths.CreateDataRequirementDataRequirementPermission.Responses.$409
    $422: Paths.CreateDataRequirementDataRequirementPermission.Responses.$422
  }

  createContactDataRequirementPermission: {
    $201: Paths.CreateContactDataRequirementPermission.Responses.$201
    $400: Paths.CreateContactDataRequirementPermission.Responses.$400
    $401: Paths.CreateContactDataRequirementPermission.Responses.$401
    $404: Paths.CreateContactDataRequirementPermission.Responses.$404
    $406: Paths.CreateContactDataRequirementPermission.Responses.$406
    $409: Paths.CreateContactDataRequirementPermission.Responses.$409
    $422: Paths.CreateContactDataRequirementPermission.Responses.$422
  }

  listKycApplicationDataRequirement: {
    $200: Paths.ListKycApplicationDataRequirement.Responses.$200
    $400: Paths.ListKycApplicationDataRequirement.Responses.$400
    $401: Paths.ListKycApplicationDataRequirement.Responses.$401
    $404: Paths.ListKycApplicationDataRequirement.Responses.$404
  }

  getDataRequirement: {
    $200: Paths.GetDataRequirement.Responses.$200
    $400: Paths.GetDataRequirement.Responses.$400
    $401: Paths.GetDataRequirement.Responses.$401
    $404: Paths.GetDataRequirement.Responses.$404
  }

  listKycEntityTemplate: {
    $200: Paths.ListKycEntityTemplate.Responses.$200
    $400: Paths.ListKycEntityTemplate.Responses.$400
    $401: Paths.ListKycEntityTemplate.Responses.$401
    $403: Paths.ListKycEntityTemplate.Responses.$403
  }

  getKycEntityTemplate: {
    $200: Paths.GetKycEntityTemplate.Responses.$200
    $400: Paths.GetKycEntityTemplate.Responses.$400
    $401: Paths.GetKycEntityTemplate.Responses.$401
    $404: Paths.GetKycEntityTemplate.Responses.$404
  }

  listKycApplicationDataComparison: {
    $200: Paths.ListKycApplicationDataComparison.Responses.$200
    $400: Paths.ListKycApplicationDataComparison.Responses.$400
    $401: Paths.ListKycApplicationDataComparison.Responses.$401
    $404: Paths.ListKycApplicationDataComparison.Responses.$404
  }

  getDataComparison: {
    $200: Paths.GetDataComparison.Responses.$200
    $400: Paths.GetDataComparison.Responses.$400
    $401: Paths.GetDataComparison.Responses.$401
    $404: Paths.GetDataComparison.Responses.$404
  }

  listProfileDataComparison: {
    $200: Paths.ListProfileDataComparison.Responses.$200
    $400: Paths.ListProfileDataComparison.Responses.$400
    $401: Paths.ListProfileDataComparison.Responses.$401
    $404: Paths.ListProfileDataComparison.Responses.$404
  }

  listDataRequirementDataComparison: {
    $200: Paths.ListDataRequirementDataComparison.Responses.$200
    $400: Paths.ListDataRequirementDataComparison.Responses.$400
    $401: Paths.ListDataRequirementDataComparison.Responses.$401
    $404: Paths.ListDataRequirementDataComparison.Responses.$404
  }

  listDataComparisonComparisonResult: {
    $200: Paths.ListDataComparisonComparisonResult.Responses.$200
    $400: Paths.ListDataComparisonComparisonResult.Responses.$400
    $401: Paths.ListDataComparisonComparisonResult.Responses.$401
    $404: Paths.ListDataComparisonComparisonResult.Responses.$404
  }

  getComparisonResult: {
    $200: Paths.GetComparisonResult.Responses.$200
    $400: Paths.GetComparisonResult.Responses.$400
    $401: Paths.GetComparisonResult.Responses.$401
    $404: Paths.GetComparisonResult.Responses.$404
  }

  listDataRequirementEvidenceSubmission: {
    $200: Paths.ListDataRequirementEvidenceSubmission.Responses.$200
    $400: Paths.ListDataRequirementEvidenceSubmission.Responses.$400
    $401: Paths.ListDataRequirementEvidenceSubmission.Responses.$401
    $404: Paths.ListDataRequirementEvidenceSubmission.Responses.$404
  }

  getEvidenceSubmission: {
    $200: Paths.GetEvidenceSubmission.Responses.$200
    $400: Paths.GetEvidenceSubmission.Responses.$400
    $401: Paths.GetEvidenceSubmission.Responses.$401
    $404: Paths.GetEvidenceSubmission.Responses.$404
  }

  updateEvidenceSubmission: {
    $200: Paths.UpdateEvidenceSubmission.Responses.$200
    $400: Paths.UpdateEvidenceSubmission.Responses.$400
    $401: Paths.UpdateEvidenceSubmission.Responses.$401
    $404: Paths.UpdateEvidenceSubmission.Responses.$404
    $406: Paths.UpdateEvidenceSubmission.Responses.$406
    $409: Paths.UpdateEvidenceSubmission.Responses.$409
  }

  updateEvidenceSubmissionReject: {
    $200: Paths.UpdateEvidenceSubmissionReject.Responses.$200
    $400: Paths.UpdateEvidenceSubmissionReject.Responses.$400
    $401: Paths.UpdateEvidenceSubmissionReject.Responses.$401
    $404: Paths.UpdateEvidenceSubmissionReject.Responses.$404
    $406: Paths.UpdateEvidenceSubmissionReject.Responses.$406
  }

  updateEvidenceSubmissionApprove: {
    $200: Paths.UpdateEvidenceSubmissionApprove.Responses.$200
    $400: Paths.UpdateEvidenceSubmissionApprove.Responses.$400
    $401: Paths.UpdateEvidenceSubmissionApprove.Responses.$401
    $404: Paths.UpdateEvidenceSubmissionApprove.Responses.$404
    $406: Paths.UpdateEvidenceSubmissionApprove.Responses.$406
  }

  updateEvidenceSubmissionReplace: {
    $200: Paths.UpdateEvidenceSubmissionReplace.Responses.$200
    $400: Paths.UpdateEvidenceSubmissionReplace.Responses.$400
    $401: Paths.UpdateEvidenceSubmissionReplace.Responses.$401
    $404: Paths.UpdateEvidenceSubmissionReplace.Responses.$404
    $406: Paths.UpdateEvidenceSubmissionReplace.Responses.$406
  }

  listEvidenceSubmissionQualitativeCheck: {
    $200: Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$200
    $400: Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$400
    $401: Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$401
    $404: Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$404
  }

  getQualitativeCheck: {
    $200: Paths.GetQualitativeCheck.Responses.$200
    $400: Paths.GetQualitativeCheck.Responses.$400
    $401: Paths.GetQualitativeCheck.Responses.$401
    $404: Paths.GetQualitativeCheck.Responses.$404
  }

  listEvidenceSubmissionRejectionReason: {
    $200: Paths.ListEvidenceSubmissionRejectionReason.Responses.$200
    $400: Paths.ListEvidenceSubmissionRejectionReason.Responses.$400
    $401: Paths.ListEvidenceSubmissionRejectionReason.Responses.$401
    $404: Paths.ListEvidenceSubmissionRejectionReason.Responses.$404
  }

  getRejectionReason: {
    $200: Paths.GetRejectionReason.Responses.$200
    $400: Paths.GetRejectionReason.Responses.$400
    $401: Paths.GetRejectionReason.Responses.$401
    $404: Paths.GetRejectionReason.Responses.$404
  }

  listEvidenceSubmissionDocument: {
    $200: Paths.ListEvidenceSubmissionDocument.Responses.$200
    $400: Paths.ListEvidenceSubmissionDocument.Responses.$400
    $401: Paths.ListEvidenceSubmissionDocument.Responses.$401
    $404: Paths.ListEvidenceSubmissionDocument.Responses.$404
  }

  createEvidenceSubmissionDocument: {
    $201: Paths.CreateEvidenceSubmissionDocument.Responses.$201
    $400: Paths.CreateEvidenceSubmissionDocument.Responses.$400
    $404: Paths.CreateEvidenceSubmissionDocument.Responses.$404
    $422: Paths.CreateEvidenceSubmissionDocument.Responses.$422
  }

  getDocument: {
    $200: Paths.GetDocument.Responses.$200
    $400: Paths.GetDocument.Responses.$400
    $401: Paths.GetDocument.Responses.$401
    $404: Paths.GetDocument.Responses.$404
  }

  listDocumentDocumentToken: {
    $200: Paths.ListDocumentDocumentToken.Responses.$200
    $400: Paths.ListDocumentDocumentToken.Responses.$400
    $401: Paths.ListDocumentDocumentToken.Responses.$401
    $404: Paths.ListDocumentDocumentToken.Responses.$404
  }

  createDocumentDocumentToken: {
    $201: Paths.CreateDocumentDocumentToken.Responses.$201
    $400: Paths.CreateDocumentDocumentToken.Responses.$400
    $401: Paths.CreateDocumentDocumentToken.Responses.$401
    $404: Paths.CreateDocumentDocumentToken.Responses.$404
    $406: Paths.CreateDocumentDocumentToken.Responses.$406
  }

  getDocumentToken: {
    $200: Paths.GetDocumentToken.Responses.$200
    $400: Paths.GetDocumentToken.Responses.$400
    $401: Paths.GetDocumentToken.Responses.$401
    $404: Paths.GetDocumentToken.Responses.$404
  }

  getDocumentFileDownload: {
    $302: Paths.GetDocumentFileDownload.Responses.$302
    $404: Paths.GetDocumentFileDownload.Responses.$404
    $406: Paths.GetDocumentFileDownload.Responses.$406
    $410: Paths.GetDocumentFileDownload.Responses.$410
  }

  getForm: {
    $200: Paths.GetForm.Responses.$200
    $400: Paths.GetForm.Responses.$400
    $401: Paths.GetForm.Responses.$401
    $404: Paths.GetForm.Responses.$404
  }

  getFormFormData: {
    $200: Paths.GetFormFormData.Responses.$200
    $400: Paths.GetFormFormData.Responses.$400
    $401: Paths.GetFormFormData.Responses.$401
    $404: Paths.GetFormFormData.Responses.$404
  }

  createForm: {
    $201: Paths.CreateForm.Responses.$201
    $404: Paths.CreateForm.Responses.$404
    $422: Paths.CreateForm.Responses.$422
  }

  getFormDataById: {
    $200: Paths.GetFormDataById.Responses.$200
    $400: Paths.GetFormDataById.Responses.$400
    $401: Paths.GetFormDataById.Responses.$401
    $404: Paths.GetFormDataById.Responses.$404
  }

  listKycApplicationRelatedEntitie: {
    $200: Paths.ListKycApplicationRelatedEntitie.Responses.$200
    $400: Paths.ListKycApplicationRelatedEntitie.Responses.$400
    $401: Paths.ListKycApplicationRelatedEntitie.Responses.$401
    $404: Paths.ListKycApplicationRelatedEntitie.Responses.$404
    $409: Paths.ListKycApplicationRelatedEntitie.Responses.$409
  }

  createKycApplicationRelatedEntitie: {
    $201: Paths.CreateKycApplicationRelatedEntitie.Responses.$201
    $400: Paths.CreateKycApplicationRelatedEntitie.Responses.$400
    $401: Paths.CreateKycApplicationRelatedEntitie.Responses.$401
    $404: Paths.CreateKycApplicationRelatedEntitie.Responses.$404
    $406: Paths.CreateKycApplicationRelatedEntitie.Responses.$406
    $409: Paths.CreateKycApplicationRelatedEntitie.Responses.$409
    $422: Paths.CreateKycApplicationRelatedEntitie.Responses.$422
  }

  getRelatedEntitie: {
    $200: Paths.GetRelatedEntitie.Responses.$200
    $400: Paths.GetRelatedEntitie.Responses.$400
    $401: Paths.GetRelatedEntitie.Responses.$401
    $404: Paths.GetRelatedEntitie.Responses.$404
  }

  deleteRelatedEntitie: {
    $204: Paths.DeleteRelatedEntitie.Responses.$204
    $400: Paths.DeleteRelatedEntitie.Responses.$400
    $401: Paths.DeleteRelatedEntitie.Responses.$401
    $404: Paths.DeleteRelatedEntitie.Responses.$404
    $422: Paths.DeleteRelatedEntitie.Responses.$422
  }

  listKycApplicationPdfReportSummarie: {
    $200: Paths.ListKycApplicationPdfReportSummarie.Responses.$200
    $400: Paths.ListKycApplicationPdfReportSummarie.Responses.$400
    $401: Paths.ListKycApplicationPdfReportSummarie.Responses.$401
    $404: Paths.ListKycApplicationPdfReportSummarie.Responses.$404
  }

  createKycApplicationPdfReportSummarie: {
    $201: Paths.CreateKycApplicationPdfReportSummarie.Responses.$201
    $400: Paths.CreateKycApplicationPdfReportSummarie.Responses.$400
    $401: Paths.CreateKycApplicationPdfReportSummarie.Responses.$401
    $404: Paths.CreateKycApplicationPdfReportSummarie.Responses.$404
    $406: Paths.CreateKycApplicationPdfReportSummarie.Responses.$406
    $429: Paths.CreateKycApplicationPdfReportSummarie.Responses.$429
  }

  getPdfReportSummarie: {
    $200: Paths.GetPdfReportSummarie.Responses.$200
    $400: Paths.GetPdfReportSummarie.Responses.$400
    $401: Paths.GetPdfReportSummarie.Responses.$401
    $404: Paths.GetPdfReportSummarie.Responses.$404
  }

  getPdfReportSummarieDownload: {
    $302: Paths.GetPdfReportSummarieDownload.Responses.$302
    $400: Paths.GetPdfReportSummarieDownload.Responses.$400
    $401: Paths.GetPdfReportSummarieDownload.Responses.$401
    $404: Paths.GetPdfReportSummarieDownload.Responses.$404
    $406: Paths.GetPdfReportSummarieDownload.Responses.$406
    $409: Paths.GetPdfReportSummarieDownload.Responses.$409
  }

  listUser: {
    $200: Paths.ListUser.Responses.$200
    $400: Paths.ListUser.Responses.$400
    $401: Paths.ListUser.Responses.$401
    $403: Paths.ListUser.Responses.$403
  }

  getUser: {
    $200: Paths.GetUser.Responses.$200
    $400: Paths.GetUser.Responses.$400
    $401: Paths.GetUser.Responses.$401
    $404: Paths.GetUser.Responses.$404
  }

  getRiskRatingEvaluation: {
    $200: Paths.GetRiskRatingEvaluation.Responses.$200
    $400: Paths.GetRiskRatingEvaluation.Responses.$400
    $401: Paths.GetRiskRatingEvaluation.Responses.$401
    $404: Paths.GetRiskRatingEvaluation.Responses.$404
  }

  listKycApplicationRiskRatingEvaluation: {
    $200: Paths.ListKycApplicationRiskRatingEvaluation.Responses.$200
    $400: Paths.ListKycApplicationRiskRatingEvaluation.Responses.$400
    $401: Paths.ListKycApplicationRiskRatingEvaluation.Responses.$401
    $404: Paths.ListKycApplicationRiskRatingEvaluation.Responses.$404
  }

  listKycApplicationRiskRating: {
    $200: Paths.ListKycApplicationRiskRating.Responses.$200
    $400: Paths.ListKycApplicationRiskRating.Responses.$400
    $401: Paths.ListKycApplicationRiskRating.Responses.$401
    $404: Paths.ListKycApplicationRiskRating.Responses.$404
  }

  createKycApplicationRiskRating: {
    $201: Paths.CreateKycApplicationRiskRating.Responses.$201
    $400: Paths.CreateKycApplicationRiskRating.Responses.$400
    $401: Paths.CreateKycApplicationRiskRating.Responses.$401
    $404: Paths.CreateKycApplicationRiskRating.Responses.$404
    $406: Paths.CreateKycApplicationRiskRating.Responses.$406
  }

  getKycApplicationRiskRating: {
    $200: Paths.GetKycApplicationRiskRating.Responses.$200
    $400: Paths.GetKycApplicationRiskRating.Responses.$400
    $401: Paths.GetKycApplicationRiskRating.Responses.$401
    $404: Paths.GetKycApplicationRiskRating.Responses.$404
  }

  updateKycApplicationRiskRating: {
    $200: Paths.UpdateKycApplicationRiskRating.Responses.$200
    $400: Paths.UpdateKycApplicationRiskRating.Responses.$400
    $401: Paths.UpdateKycApplicationRiskRating.Responses.$401
    $404: Paths.UpdateKycApplicationRiskRating.Responses.$404
    $406: Paths.UpdateKycApplicationRiskRating.Responses.$406
  }

  listKycApplicationRiskRatingsQuestion: {
    $200: Paths.ListKycApplicationRiskRatingsQuestion.Responses.$200
    $404: Paths.ListKycApplicationRiskRatingsQuestion.Responses.$404
  }

  getBankAccountSchema: {
    $200: Paths.GetBankAccountSchema.Responses.$200
    $400: Paths.GetBankAccountSchema.Responses.$400
    $401: Paths.GetBankAccountSchema.Responses.$401
    $404: Paths.GetBankAccountSchema.Responses.$404
  }

  listKycApplicationBankAccount: {
    $200: Paths.ListKycApplicationBankAccount.Responses.$200
    $400: Paths.ListKycApplicationBankAccount.Responses.$400
    $401: Paths.ListKycApplicationBankAccount.Responses.$401
    $404: Paths.ListKycApplicationBankAccount.Responses.$404
  }

  createKycApplicationBankAccount: {
    $201: Paths.CreateKycApplicationBankAccount.Responses.$201
    $400: Paths.CreateKycApplicationBankAccount.Responses.$400
    $401: Paths.CreateKycApplicationBankAccount.Responses.$401
    $404: Paths.CreateKycApplicationBankAccount.Responses.$404
    $406: Paths.CreateKycApplicationBankAccount.Responses.$406
    $422: Paths.CreateKycApplicationBankAccount.Responses.$422
  }

  getBankAccount: {
    $200: Paths.GetBankAccount.Responses.$200
    $400: Paths.GetBankAccount.Responses.$400
    $401: Paths.GetBankAccount.Responses.$401
    $404: Paths.GetBankAccount.Responses.$404
  }

  getBankAccountVerification: {
    $200: Paths.GetBankAccountVerification.Responses.$200
    $400: Paths.GetBankAccountVerification.Responses.$400
    $401: Paths.GetBankAccountVerification.Responses.$401
    $404: Paths.GetBankAccountVerification.Responses.$404
  }

  updateBankAccountVerification: {
    $200: Paths.UpdateBankAccountVerification.Responses.$200
    $400: Paths.UpdateBankAccountVerification.Responses.$400
    $401: Paths.UpdateBankAccountVerification.Responses.$401
    $404: Paths.UpdateBankAccountVerification.Responses.$404
    $406: Paths.UpdateBankAccountVerification.Responses.$406
  }

  listBankAccountBankAccountVerification: {
    $200: Paths.ListBankAccountBankAccountVerification.Responses.$200
    $400: Paths.ListBankAccountBankAccountVerification.Responses.$400
    $401: Paths.ListBankAccountBankAccountVerification.Responses.$401
    $404: Paths.ListBankAccountBankAccountVerification.Responses.$404
  }

  listKycApplicationConsumerReportQuerie: {
    $200: Paths.ListKycApplicationConsumerReportQuerie.Responses.$200
    $400: Paths.ListKycApplicationConsumerReportQuerie.Responses.$400
    $401: Paths.ListKycApplicationConsumerReportQuerie.Responses.$401
    $404: Paths.ListKycApplicationConsumerReportQuerie.Responses.$404
  }

  getConsumerReportQuerie: {
    $200: Paths.GetConsumerReportQuerie.Responses.$200
    $400: Paths.GetConsumerReportQuerie.Responses.$400
    $401: Paths.GetConsumerReportQuerie.Responses.$401
    $404: Paths.GetConsumerReportQuerie.Responses.$404
  }

  getConsumerReportQuerieDownload: {
    $302: Paths.GetConsumerReportQuerieDownload.Responses.$302
    $400: Paths.GetConsumerReportQuerieDownload.Responses.$400
    $401: Paths.GetConsumerReportQuerieDownload.Responses.$401
    $404: Paths.GetConsumerReportQuerieDownload.Responses.$404
    $406: Paths.GetConsumerReportQuerieDownload.Responses.$406
    $409: Paths.GetConsumerReportQuerieDownload.Responses.$409
  }

  createOnboardingInvitation: {
    $201: Paths.CreateOnboardingInvitation.Responses.$201
    $400: Paths.CreateOnboardingInvitation.Responses.$400
    $401: Paths.CreateOnboardingInvitation.Responses.$401
    $403: Paths.CreateOnboardingInvitation.Responses.$403
    $406: Paths.CreateOnboardingInvitation.Responses.$406
  }

  updateOnboardingInvitation: {
    $200: Paths.UpdateOnboardingInvitation.Responses.$200
    $400: Paths.UpdateOnboardingInvitation.Responses.$400
    $401: Paths.UpdateOnboardingInvitation.Responses.$401
    $404: Paths.UpdateOnboardingInvitation.Responses.$404
    $406: Paths.UpdateOnboardingInvitation.Responses.$406
    $422: Paths.UpdateOnboardingInvitation.Responses.$422
  }

  listFormFormPdfExport: {
    $200: Paths.ListFormFormPdfExport.Responses.$200
    $400: Paths.ListFormFormPdfExport.Responses.$400
    $401: Paths.ListFormFormPdfExport.Responses.$401
    $404: Paths.ListFormFormPdfExport.Responses.$404
  }

  createFormFormPdfExport: {
    $201: Paths.CreateFormFormPdfExport.Responses.$201
    $400: Paths.CreateFormFormPdfExport.Responses.$400
    $401: Paths.CreateFormFormPdfExport.Responses.$401
    $404: Paths.CreateFormFormPdfExport.Responses.$404
    $406: Paths.CreateFormFormPdfExport.Responses.$406
    $429: Paths.CreateFormFormPdfExport.Responses.$429
  }

  getFormPdfExport: {
    $200: Paths.GetFormPdfExport.Responses.$200
    $400: Paths.GetFormPdfExport.Responses.$400
    $401: Paths.GetFormPdfExport.Responses.$401
    $404: Paths.GetFormPdfExport.Responses.$404
  }

  getFormPdfExportDownload: {
    $302: Paths.GetFormPdfExportDownload.Responses.$302
    $400: Paths.GetFormPdfExportDownload.Responses.$400
    $401: Paths.GetFormPdfExportDownload.Responses.$401
    $404: Paths.GetFormPdfExportDownload.Responses.$404
    $406: Paths.GetFormPdfExportDownload.Responses.$406
    $409: Paths.GetFormPdfExportDownload.Responses.$409
  }

  listKycEntityTemplateDesignationTemplate: {
    $200: Paths.ListKycEntityTemplateDesignationTemplate.Responses.$200
    $400: Paths.ListKycEntityTemplateDesignationTemplate.Responses.$400
    $401: Paths.ListKycEntityTemplateDesignationTemplate.Responses.$401
    $404: Paths.ListKycEntityTemplateDesignationTemplate.Responses.$404
  }

  getRelatedEntityDesignation: {
    $200: Paths.GetRelatedEntityDesignation.Responses.$200
    $400: Paths.GetRelatedEntityDesignation.Responses.$400
    $401: Paths.GetRelatedEntityDesignation.Responses.$401
    $404: Paths.GetRelatedEntityDesignation.Responses.$404
  }

  deleteRelatedEntityDesignation: {
    $204: Paths.DeleteRelatedEntityDesignation.Responses.$204
    $400: Paths.DeleteRelatedEntityDesignation.Responses.$400
    $401: Paths.DeleteRelatedEntityDesignation.Responses.$401
    $404: Paths.DeleteRelatedEntityDesignation.Responses.$404
  }

  listRelatedEntitieRelatedEntityDesignation: {
    $200: Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$200
    $400: Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$400
    $401: Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$401
    $404: Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$404
  }

  createRelatedEntitieRelatedEntityDesignation: {
    $201: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$201
    $400: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$400
    $401: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$401
    $404: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$404
    $406: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$406
    $422: Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$422
  }

  listProductProductProductAddon: {
    $200: Paths.ListProductProductProductAddon.Responses.$200
    $400: Paths.ListProductProductProductAddon.Responses.$400
    $401: Paths.ListProductProductProductAddon.Responses.$401
    $404: Paths.ListProductProductProductAddon.Responses.$404
  }

  listAccountApplicationAccountAddon: {
    $200: Paths.ListAccountApplicationAccountAddon.Responses.$200
    $400: Paths.ListAccountApplicationAccountAddon.Responses.$400
    $401: Paths.ListAccountApplicationAccountAddon.Responses.$401
    $404: Paths.ListAccountApplicationAccountAddon.Responses.$404
  }

  getAccountAddon: {
    $200: Paths.GetAccountAddon.Responses.$200
    $400: Paths.GetAccountAddon.Responses.$400
    $401: Paths.GetAccountAddon.Responses.$401
    $404: Paths.GetAccountAddon.Responses.$404
  }

  listProductDisclosure: {
    $200: Paths.ListProductDisclosure.Responses.$200
    $400: Paths.ListProductDisclosure.Responses.$400
    $401: Paths.ListProductDisclosure.Responses.$401
    $403: Paths.ListProductDisclosure.Responses.$403
    $404: Paths.ListProductDisclosure.Responses.$404
  }

  listDisclosure: {
    $200: Paths.ListDisclosure.Responses.$200
    $400: Paths.ListDisclosure.Responses.$400
    $401: Paths.ListDisclosure.Responses.$401
    $403: Paths.ListDisclosure.Responses.$403
  }

  getDisclosure: {
    $200: Paths.GetDisclosure.Responses.$200
    $400: Paths.GetDisclosure.Responses.$400
    $401: Paths.GetDisclosure.Responses.$401
    $404: Paths.GetDisclosure.Responses.$404
  }

  listProduct: {
    $200: Paths.ListProduct.Responses.$200
    $400: Paths.ListProduct.Responses.$400
    $401: Paths.ListProduct.Responses.$401
    $403: Paths.ListProduct.Responses.$403
  }

  getProduct: {
    $200: Paths.GetProduct.Responses.$200
    $400: Paths.GetProduct.Responses.$400
    $401: Paths.GetProduct.Responses.$401
    $404: Paths.GetProduct.Responses.$404
  }

  listKycEntityTemplateProduct: {
    $200: Paths.ListKycEntityTemplateProduct.Responses.$200
    $400: Paths.ListKycEntityTemplateProduct.Responses.$400
    $401: Paths.ListKycEntityTemplateProduct.Responses.$401
    $404: Paths.ListKycEntityTemplateProduct.Responses.$404
  }

  getAccountApplicationAccountSummaryPdf: {
    $200: Paths.GetAccountApplicationAccountSummaryPdf.Responses.$200
    $404: Paths.GetAccountApplicationAccountSummaryPdf.Responses.$404
  }

  listKycEntityTemplateDisclosure: {
    $200: Paths.ListKycEntityTemplateDisclosure.Responses.$200
    $400: Paths.ListKycEntityTemplateDisclosure.Responses.$400
    $401: Paths.ListKycEntityTemplateDisclosure.Responses.$401
    $403: Paths.ListKycEntityTemplateDisclosure.Responses.$403
    $404: Paths.ListKycEntityTemplateDisclosure.Responses.$404
  }

  listAccountApplicationDataRequirement: {
    $200: Paths.ListAccountApplicationDataRequirement.Responses.$200
    $400: Paths.ListAccountApplicationDataRequirement.Responses.$400
    $401: Paths.ListAccountApplicationDataRequirement.Responses.$401
    $404: Paths.ListAccountApplicationDataRequirement.Responses.$404
  }

  listProductProductAddonsDisclosure: {
    $200: Paths.ListProductProductAddonsDisclosure.Responses.$200
    $400: Paths.ListProductProductAddonsDisclosure.Responses.$400
    $401: Paths.ListProductProductAddonsDisclosure.Responses.$401
    $403: Paths.ListProductProductAddonsDisclosure.Responses.$403
  }

  getAccountApplicationAccountSummary: {
    $200: Paths.GetAccountApplicationAccountSummary.Responses.$200
    $404: Paths.GetAccountApplicationAccountSummary.Responses.$404
  }

  listKycApplicationAccountApplication: {
    $200: Paths.ListKycApplicationAccountApplication.Responses.$200
    $400: Paths.ListKycApplicationAccountApplication.Responses.$400
    $401: Paths.ListKycApplicationAccountApplication.Responses.$401
    $404: Paths.ListKycApplicationAccountApplication.Responses.$404
  }

  createKycApplicationAccountApplication: {
    $201: Paths.CreateKycApplicationAccountApplication.Responses.$201
    $400: Paths.CreateKycApplicationAccountApplication.Responses.$400
    $401: Paths.CreateKycApplicationAccountApplication.Responses.$401
    $404: Paths.CreateKycApplicationAccountApplication.Responses.$404
    $406: Paths.CreateKycApplicationAccountApplication.Responses.$406
  }

  getAccountApplication: {
    $200: Paths.GetAccountApplication.Responses.$200
    $400: Paths.GetAccountApplication.Responses.$400
    $401: Paths.GetAccountApplication.Responses.$401
    $404: Paths.GetAccountApplication.Responses.$404
  }

  updateAccountApplication: {
    $200: Paths.UpdateAccountApplication.Responses.$200
    $400: Paths.UpdateAccountApplication.Responses.$400
    $401: Paths.UpdateAccountApplication.Responses.$401
    $404: Paths.UpdateAccountApplication.Responses.$404
    $406: Paths.UpdateAccountApplication.Responses.$406
  }

  updateAccountApplicationDisclosureAcceptances: {
    $200: Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$200
    $400: Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$400
    $401: Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$401
    $404: Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$404
    $406: Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$406
  }
}

export interface PathsDictionary {
  ['/api/v2/authentications/new']: {
    'get'(
      parameters?: Parameters<Paths.RequestAuthentication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.RequestAuthentication.Responses.$200
      | Paths.RequestAuthentication.Responses.$400
      | Paths.RequestAuthentication.Responses.$401
    >
  }
  ['/api/v2/tokens/new']: {
    'get'(
      parameters?: Parameters<Paths.RequestToken.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.RequestToken.Responses.$200
      | Paths.RequestToken.Responses.$400
      | Paths.RequestToken.Responses.$401
    >
  }
  ['/api/v2/kyc_applications']: {
    'get'(
      parameters?: Parameters<Paths.ListKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplication.Responses.$200
      | Paths.ListKycApplication.Responses.$400
      | Paths.ListKycApplication.Responses.$401
      | Paths.ListKycApplication.Responses.$403
    >

    'post'(
      parameters?: Parameters<Paths.CreateKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplication.Responses.$201
      | Paths.CreateKycApplication.Responses.$400
      | Paths.CreateKycApplication.Responses.$401
      | Paths.CreateKycApplication.Responses.$406
      | Paths.CreateKycApplication.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetKycApplication.Responses.$200
      | Paths.GetKycApplication.Responses.$400
      | Paths.GetKycApplication.Responses.$401
      | Paths.GetKycApplication.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteKycApplication.Responses.$204
      | Paths.DeleteKycApplication.Responses.$400
      | Paths.DeleteKycApplication.Responses.$401
      | Paths.DeleteKycApplication.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/approve']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateKycApplicationApprove.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateKycApplicationApprove.Responses.$200
      | Paths.UpdateKycApplicationApprove.Responses.$400
      | Paths.UpdateKycApplicationApprove.Responses.$401
      | Paths.UpdateKycApplicationApprove.Responses.$404
      | Paths.UpdateKycApplicationApprove.Responses.$406
      | Paths.UpdateKycApplicationApprove.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/unapprove']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateKycApplicationUnapprove.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateKycApplicationUnapprove.Responses.$200
      | Paths.UpdateKycApplicationUnapprove.Responses.$400
      | Paths.UpdateKycApplicationUnapprove.Responses.$401
      | Paths.UpdateKycApplicationUnapprove.Responses.$404
      | Paths.UpdateKycApplicationUnapprove.Responses.$406
      | Paths.UpdateKycApplicationUnapprove.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/transfer']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateKycApplicationTransfer.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateKycApplicationTransfer.Responses.$200
      | Paths.UpdateKycApplicationTransfer.Responses.$400
      | Paths.UpdateKycApplicationTransfer.Responses.$401
      | Paths.UpdateKycApplicationTransfer.Responses.$404
      | Paths.UpdateKycApplicationTransfer.Responses.$406
      | Paths.UpdateKycApplicationTransfer.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/archive']: {
    'post'(
      parameters?: Parameters<Paths.ArchiveKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      Paths.ArchiveKycApplication.Responses.$204 | Paths.ArchiveKycApplication.Responses.$400
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/unarchive']: {
    'post'(
      parameters?: Parameters<Paths.UnarchiveKycApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      Paths.UnarchiveKycApplication.Responses.$200 | Paths.UnarchiveKycApplication.Responses.$400
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/status_summaries']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationStatusSummarie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationStatusSummarie.Responses.$200
      | Paths.ListKycApplicationStatusSummarie.Responses.$400
      | Paths.ListKycApplicationStatusSummarie.Responses.$401
      | Paths.ListKycApplicationStatusSummarie.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/profiles']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationProfile.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationProfile.Responses.$200
      | Paths.ListKycApplicationProfile.Responses.$400
      | Paths.ListKycApplicationProfile.Responses.$401
      | Paths.ListKycApplicationProfile.Responses.$404
    >
  }
  ['/api/v2/profiles/{profile_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetProfile.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetProfile.Responses.$200
      | Paths.GetProfile.Responses.$400
      | Paths.GetProfile.Responses.$401
      | Paths.GetProfile.Responses.$404
    >
  }
  ['/api/v2/profiles/{profile_id}/names']: {
    'get'(
      parameters?: Parameters<Paths.ListProfileName.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileName.Responses.$200
      | Paths.ListProfileName.Responses.$400
      | Paths.ListProfileName.Responses.$401
      | Paths.ListProfileName.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateProfileName.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateProfileName.Responses.$201
      | Paths.CreateProfileName.Responses.$400
      | Paths.CreateProfileName.Responses.$401
      | Paths.CreateProfileName.Responses.$404
      | Paths.CreateProfileName.Responses.$406
      | Paths.CreateProfileName.Responses.$422
    >
  }
  ['/api/v2/names/{name_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetName.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetName.Responses.$200
      | Paths.GetName.Responses.$400
      | Paths.GetName.Responses.$401
      | Paths.GetName.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteName.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteName.Responses.$204
      | Paths.DeleteName.Responses.$400
      | Paths.DeleteName.Responses.$401
      | Paths.DeleteName.Responses.$404
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateName.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateName.Responses.$200
      | Paths.UpdateName.Responses.$400
      | Paths.UpdateName.Responses.$401
      | Paths.UpdateName.Responses.$404
      | Paths.UpdateName.Responses.$406
      | Paths.UpdateName.Responses.$422
    >
  }
  ['/api/v2/profiles/{profile_id}/numbers']: {
    'get'(
      parameters?: Parameters<Paths.ListProfileNumber.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileNumber.Responses.$200
      | Paths.ListProfileNumber.Responses.$400
      | Paths.ListProfileNumber.Responses.$401
      | Paths.ListProfileNumber.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateProfileNumber.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateProfileNumber.Responses.$201
      | Paths.CreateProfileNumber.Responses.$400
      | Paths.CreateProfileNumber.Responses.$401
      | Paths.CreateProfileNumber.Responses.$404
      | Paths.CreateProfileNumber.Responses.$406
      | Paths.CreateProfileNumber.Responses.$422
    >
  }
  ['/api/v2/numbers/{number_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetNumber.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetNumber.Responses.$200
      | Paths.GetNumber.Responses.$400
      | Paths.GetNumber.Responses.$401
      | Paths.GetNumber.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteNumber.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteNumber.Responses.$204
      | Paths.DeleteNumber.Responses.$400
      | Paths.DeleteNumber.Responses.$401
      | Paths.DeleteNumber.Responses.$404
      | Paths.DeleteNumber.Responses.$422
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateNumber.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateNumber.Responses.$200
      | Paths.UpdateNumber.Responses.$400
      | Paths.UpdateNumber.Responses.$401
      | Paths.UpdateNumber.Responses.$404
      | Paths.UpdateNumber.Responses.$406
      | Paths.UpdateNumber.Responses.$422
    >
  }
  ['/api/v2/profiles/{profile_id}/contacts']: {
    'get'(
      parameters?: Parameters<Paths.ListProfileContact.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileContact.Responses.$200
      | Paths.ListProfileContact.Responses.$400
      | Paths.ListProfileContact.Responses.$401
      | Paths.ListProfileContact.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateProfileContact.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateProfileContact.Responses.$201
      | Paths.CreateProfileContact.Responses.$400
      | Paths.CreateProfileContact.Responses.$401
      | Paths.CreateProfileContact.Responses.$404
      | Paths.CreateProfileContact.Responses.$406
      | Paths.CreateProfileContact.Responses.$422
    >
  }
  ['/api/v2/contacts/{contact_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetContact.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetContact.Responses.$200
      | Paths.GetContact.Responses.$400
      | Paths.GetContact.Responses.$401
      | Paths.GetContact.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteContact.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteContact.Responses.$204
      | Paths.DeleteContact.Responses.$400
      | Paths.DeleteContact.Responses.$401
      | Paths.DeleteContact.Responses.$404
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateContact.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateContact.Responses.$200
      | Paths.UpdateContact.Responses.$400
      | Paths.UpdateContact.Responses.$401
      | Paths.UpdateContact.Responses.$404
      | Paths.UpdateContact.Responses.$406
      | Paths.UpdateContact.Responses.$422
    >
  }
  ['/api/v2/profiles/{profile_id}/addresses']: {
    'get'(
      parameters?: Parameters<Paths.ListProfileAddresse.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileAddresse.Responses.$200
      | Paths.ListProfileAddresse.Responses.$400
      | Paths.ListProfileAddresse.Responses.$401
      | Paths.ListProfileAddresse.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateProfileAddresse.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateProfileAddresse.Responses.$201
      | Paths.CreateProfileAddresse.Responses.$400
      | Paths.CreateProfileAddresse.Responses.$401
      | Paths.CreateProfileAddresse.Responses.$404
      | Paths.CreateProfileAddresse.Responses.$406
      | Paths.CreateProfileAddresse.Responses.$422
    >
  }
  ['/api/v2/addresses/{address_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetAddresse.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAddresse.Responses.$200
      | Paths.GetAddresse.Responses.$400
      | Paths.GetAddresse.Responses.$401
      | Paths.GetAddresse.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteAddresse.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteAddresse.Responses.$204
      | Paths.DeleteAddresse.Responses.$400
      | Paths.DeleteAddresse.Responses.$401
      | Paths.DeleteAddresse.Responses.$404
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateAddresse.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateAddresse.Responses.$200
      | Paths.UpdateAddresse.Responses.$400
      | Paths.UpdateAddresse.Responses.$401
      | Paths.UpdateAddresse.Responses.$404
      | Paths.UpdateAddresse.Responses.$406
      | Paths.UpdateAddresse.Responses.$422
    >
  }
  ['/api/v2/profiles/{profile_id}/additional_details']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListProfileAdditionalDetail.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileAdditionalDetail.Responses.$200
      | Paths.ListProfileAdditionalDetail.Responses.$400
      | Paths.ListProfileAdditionalDetail.Responses.$401
      | Paths.ListProfileAdditionalDetail.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateProfileAdditionalDetail.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateProfileAdditionalDetail.Responses.$201
      | Paths.CreateProfileAdditionalDetail.Responses.$400
      | Paths.CreateProfileAdditionalDetail.Responses.$401
      | Paths.CreateProfileAdditionalDetail.Responses.$404
      | Paths.CreateProfileAdditionalDetail.Responses.$406
      | Paths.CreateProfileAdditionalDetail.Responses.$422
    >
  }
  ['/api/v2/additional_details/{additional_detail_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetAdditionalDetail.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAdditionalDetail.Responses.$200
      | Paths.GetAdditionalDetail.Responses.$400
      | Paths.GetAdditionalDetail.Responses.$401
      | Paths.GetAdditionalDetail.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteAdditionalDetail.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteAdditionalDetail.Responses.$204
      | Paths.DeleteAdditionalDetail.Responses.$400
      | Paths.DeleteAdditionalDetail.Responses.$401
      | Paths.DeleteAdditionalDetail.Responses.$404
      | Paths.DeleteAdditionalDetail.Responses.$422
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateAdditionalDetail.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateAdditionalDetail.Responses.$200
      | Paths.UpdateAdditionalDetail.Responses.$400
      | Paths.UpdateAdditionalDetail.Responses.$401
      | Paths.UpdateAdditionalDetail.Responses.$404
      | Paths.UpdateAdditionalDetail.Responses.$406
      | Paths.UpdateAdditionalDetail.Responses.$422
    >
  }
  ['/api/v2/contacts/{contact_information_id}/invitation_tokens']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListContactInvitationToken.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListContactInvitationToken.Responses.$200
      | Paths.ListContactInvitationToken.Responses.$400
      | Paths.ListContactInvitationToken.Responses.$401
      | Paths.ListContactInvitationToken.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateContactInvitationToken.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateContactInvitationToken.Responses.$201
      | Paths.CreateContactInvitationToken.Responses.$400
      | Paths.CreateContactInvitationToken.Responses.$401
      | Paths.CreateContactInvitationToken.Responses.$404
      | Paths.CreateContactInvitationToken.Responses.$406
      | Paths.CreateContactInvitationToken.Responses.$409
      | Paths.CreateContactInvitationToken.Responses.$422
    >
  }
  ['/api/v2/invitation_tokens/{invitation_token_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetInvitationToken.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetInvitationToken.Responses.$200
      | Paths.GetInvitationToken.Responses.$400
      | Paths.GetInvitationToken.Responses.$401
      | Paths.GetInvitationToken.Responses.$404
    >

    'patch'(
      parameters?: Parameters<Paths.UpdateInvitationToken.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateInvitationToken.Responses.$200
      | Paths.UpdateInvitationToken.Responses.$400
      | Paths.UpdateInvitationToken.Responses.$401
      | Paths.UpdateInvitationToken.Responses.$404
      | Paths.UpdateInvitationToken.Responses.$406
      | Paths.UpdateInvitationToken.Responses.$409
    >
  }
  ['/api/v2/contacts/{contact_information_id}/data_requirement_permissions']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListContactDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListContactDataRequirementPermission.Responses.$200
      | Paths.ListContactDataRequirementPermission.Responses.$400
      | Paths.ListContactDataRequirementPermission.Responses.$401
      | Paths.ListContactDataRequirementPermission.Responses.$404
    >
  }
  ['/api/v2/data_requirement_permissions/{data_requirement_permission_id}']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDataRequirementPermission.Responses.$200
      | Paths.GetDataRequirementPermission.Responses.$400
      | Paths.GetDataRequirementPermission.Responses.$401
      | Paths.GetDataRequirementPermission.Responses.$404
    >

    'delete'(
      parameters?:
        | Parameters<Paths.DeleteDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteDataRequirementPermission.Responses.$204
      | Paths.DeleteDataRequirementPermission.Responses.$400
      | Paths.DeleteDataRequirementPermission.Responses.$401
      | Paths.DeleteDataRequirementPermission.Responses.$404
    >
  }
  ['/api/v2/data_requirement_permissions']: {
    'post'(
      parameters?:
        | Parameters<Paths.CreateDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateDataRequirementPermission.Responses.$201
      | Paths.CreateDataRequirementPermission.Responses.$400
      | Paths.CreateDataRequirementPermission.Responses.$401
      | Paths.CreateDataRequirementPermission.Responses.$404
      | Paths.CreateDataRequirementPermission.Responses.$406
      | Paths.CreateDataRequirementPermission.Responses.$409
      | Paths.CreateDataRequirementPermission.Responses.$422
    >
  }
  ['/api/v2/data_requirements/{data_requirement_id}/data_requirement_permissions']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListDataRequirementDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDataRequirementDataRequirementPermission.Responses.$200
      | Paths.ListDataRequirementDataRequirementPermission.Responses.$400
      | Paths.ListDataRequirementDataRequirementPermission.Responses.$401
      | Paths.ListDataRequirementDataRequirementPermission.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateDataRequirementDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$201
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$400
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$401
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$404
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$406
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$409
      | Paths.CreateDataRequirementDataRequirementPermission.Responses.$422
    >
  }
  ['/api/v2/contacts/{contact_id}/data_requirement_permissions']: {
    'post'(
      parameters?:
        | Parameters<Paths.CreateContactDataRequirementPermission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateContactDataRequirementPermission.Responses.$201
      | Paths.CreateContactDataRequirementPermission.Responses.$400
      | Paths.CreateContactDataRequirementPermission.Responses.$401
      | Paths.CreateContactDataRequirementPermission.Responses.$404
      | Paths.CreateContactDataRequirementPermission.Responses.$406
      | Paths.CreateContactDataRequirementPermission.Responses.$409
      | Paths.CreateContactDataRequirementPermission.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/data_requirements']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationDataRequirement.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationDataRequirement.Responses.$200
      | Paths.ListKycApplicationDataRequirement.Responses.$400
      | Paths.ListKycApplicationDataRequirement.Responses.$401
      | Paths.ListKycApplicationDataRequirement.Responses.$404
    >
  }
  ['/api/v2/data_requirements/{data_requirement_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDataRequirement.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDataRequirement.Responses.$200
      | Paths.GetDataRequirement.Responses.$400
      | Paths.GetDataRequirement.Responses.$401
      | Paths.GetDataRequirement.Responses.$404
    >
  }
  ['/api/v2/kyc_entity_templates']: {
    'get'(
      parameters?: Parameters<Paths.ListKycEntityTemplate.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycEntityTemplate.Responses.$200
      | Paths.ListKycEntityTemplate.Responses.$400
      | Paths.ListKycEntityTemplate.Responses.$401
      | Paths.ListKycEntityTemplate.Responses.$403
    >
  }
  ['/api/v2/kyc_entity_templates/{kyc_entity_template_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetKycEntityTemplate.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetKycEntityTemplate.Responses.$200
      | Paths.GetKycEntityTemplate.Responses.$400
      | Paths.GetKycEntityTemplate.Responses.$401
      | Paths.GetKycEntityTemplate.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/data_comparisons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationDataComparison.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationDataComparison.Responses.$200
      | Paths.ListKycApplicationDataComparison.Responses.$400
      | Paths.ListKycApplicationDataComparison.Responses.$401
      | Paths.ListKycApplicationDataComparison.Responses.$404
    >
  }
  ['/api/v2/data_comparisons/{data_comparison_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDataComparison.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDataComparison.Responses.$200
      | Paths.GetDataComparison.Responses.$400
      | Paths.GetDataComparison.Responses.$401
      | Paths.GetDataComparison.Responses.$404
    >
  }
  ['/api/v2/profiles/{profile_id}/data_comparisons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListProfileDataComparison.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProfileDataComparison.Responses.$200
      | Paths.ListProfileDataComparison.Responses.$400
      | Paths.ListProfileDataComparison.Responses.$401
      | Paths.ListProfileDataComparison.Responses.$404
    >
  }
  ['/api/v2/data_requirements/{data_requirement_id}/data_comparisons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListDataRequirementDataComparison.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDataRequirementDataComparison.Responses.$200
      | Paths.ListDataRequirementDataComparison.Responses.$400
      | Paths.ListDataRequirementDataComparison.Responses.$401
      | Paths.ListDataRequirementDataComparison.Responses.$404
    >
  }
  ['/api/v2/data_comparisons/{data_comparison_id}/comparison_results']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListDataComparisonComparisonResult.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDataComparisonComparisonResult.Responses.$200
      | Paths.ListDataComparisonComparisonResult.Responses.$400
      | Paths.ListDataComparisonComparisonResult.Responses.$401
      | Paths.ListDataComparisonComparisonResult.Responses.$404
    >
  }
  ['/api/v2/comparison_results/{comparison_result_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetComparisonResult.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetComparisonResult.Responses.$200
      | Paths.GetComparisonResult.Responses.$400
      | Paths.GetComparisonResult.Responses.$401
      | Paths.GetComparisonResult.Responses.$404
    >
  }
  ['/api/v2/data_requirements/{data_requirement_id}/evidence_submissions']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListDataRequirementEvidenceSubmission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDataRequirementEvidenceSubmission.Responses.$200
      | Paths.ListDataRequirementEvidenceSubmission.Responses.$400
      | Paths.ListDataRequirementEvidenceSubmission.Responses.$401
      | Paths.ListDataRequirementEvidenceSubmission.Responses.$404
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetEvidenceSubmission.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetEvidenceSubmission.Responses.$200
      | Paths.GetEvidenceSubmission.Responses.$400
      | Paths.GetEvidenceSubmission.Responses.$401
      | Paths.GetEvidenceSubmission.Responses.$404
    >

    'patch'(
      parameters?:
        | Parameters<Paths.UpdateEvidenceSubmission.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateEvidenceSubmission.Responses.$200
      | Paths.UpdateEvidenceSubmission.Responses.$400
      | Paths.UpdateEvidenceSubmission.Responses.$401
      | Paths.UpdateEvidenceSubmission.Responses.$404
      | Paths.UpdateEvidenceSubmission.Responses.$406
      | Paths.UpdateEvidenceSubmission.Responses.$409
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/reject']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateEvidenceSubmissionReject.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateEvidenceSubmissionReject.Responses.$200
      | Paths.UpdateEvidenceSubmissionReject.Responses.$400
      | Paths.UpdateEvidenceSubmissionReject.Responses.$401
      | Paths.UpdateEvidenceSubmissionReject.Responses.$404
      | Paths.UpdateEvidenceSubmissionReject.Responses.$406
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/approve']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateEvidenceSubmissionApprove.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateEvidenceSubmissionApprove.Responses.$200
      | Paths.UpdateEvidenceSubmissionApprove.Responses.$400
      | Paths.UpdateEvidenceSubmissionApprove.Responses.$401
      | Paths.UpdateEvidenceSubmissionApprove.Responses.$404
      | Paths.UpdateEvidenceSubmissionApprove.Responses.$406
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/replace']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateEvidenceSubmissionReplace.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateEvidenceSubmissionReplace.Responses.$200
      | Paths.UpdateEvidenceSubmissionReplace.Responses.$400
      | Paths.UpdateEvidenceSubmissionReplace.Responses.$401
      | Paths.UpdateEvidenceSubmissionReplace.Responses.$404
      | Paths.UpdateEvidenceSubmissionReplace.Responses.$406
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/qualitative_checks']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListEvidenceSubmissionQualitativeCheck.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$200
      | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$400
      | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$401
      | Paths.ListEvidenceSubmissionQualitativeCheck.Responses.$404
    >
  }
  ['/api/v2/qualitative_checks/{qualitative_check_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetQualitativeCheck.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetQualitativeCheck.Responses.$200
      | Paths.GetQualitativeCheck.Responses.$400
      | Paths.GetQualitativeCheck.Responses.$401
      | Paths.GetQualitativeCheck.Responses.$404
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/rejection_reasons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListEvidenceSubmissionRejectionReason.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListEvidenceSubmissionRejectionReason.Responses.$200
      | Paths.ListEvidenceSubmissionRejectionReason.Responses.$400
      | Paths.ListEvidenceSubmissionRejectionReason.Responses.$401
      | Paths.ListEvidenceSubmissionRejectionReason.Responses.$404
    >
  }
  ['/api/v2/rejection_reasons/{rejection_reason_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetRejectionReason.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetRejectionReason.Responses.$200
      | Paths.GetRejectionReason.Responses.$400
      | Paths.GetRejectionReason.Responses.$401
      | Paths.GetRejectionReason.Responses.$404
    >
  }
  ['/api/v2/evidence_submissions/{evidence_submission_id}/documents']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListEvidenceSubmissionDocument.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListEvidenceSubmissionDocument.Responses.$200
      | Paths.ListEvidenceSubmissionDocument.Responses.$400
      | Paths.ListEvidenceSubmissionDocument.Responses.$401
      | Paths.ListEvidenceSubmissionDocument.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateEvidenceSubmissionDocument.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateEvidenceSubmissionDocument.Responses.$201
      | Paths.CreateEvidenceSubmissionDocument.Responses.$400
      | Paths.CreateEvidenceSubmissionDocument.Responses.$404
      | Paths.CreateEvidenceSubmissionDocument.Responses.$422
    >
  }
  ['/api/v2/documents/{document_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDocument.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDocument.Responses.$200
      | Paths.GetDocument.Responses.$400
      | Paths.GetDocument.Responses.$401
      | Paths.GetDocument.Responses.$404
    >
  }
  ['/api/v2/documents/{document_id}/document_tokens']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListDocumentDocumentToken.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDocumentDocumentToken.Responses.$200
      | Paths.ListDocumentDocumentToken.Responses.$400
      | Paths.ListDocumentDocumentToken.Responses.$401
      | Paths.ListDocumentDocumentToken.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateDocumentDocumentToken.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateDocumentDocumentToken.Responses.$201
      | Paths.CreateDocumentDocumentToken.Responses.$400
      | Paths.CreateDocumentDocumentToken.Responses.$401
      | Paths.CreateDocumentDocumentToken.Responses.$404
      | Paths.CreateDocumentDocumentToken.Responses.$406
    >
  }
  ['/api/v2/document_tokens/{document_token_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDocumentToken.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDocumentToken.Responses.$200
      | Paths.GetDocumentToken.Responses.$400
      | Paths.GetDocumentToken.Responses.$401
      | Paths.GetDocumentToken.Responses.$404
    >
  }
  ['/api/v2/document_file_downloads/{document_token_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDocumentFileDownload.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDocumentFileDownload.Responses.$302
      | Paths.GetDocumentFileDownload.Responses.$404
      | Paths.GetDocumentFileDownload.Responses.$406
      | Paths.GetDocumentFileDownload.Responses.$410
    >
  }
  ['/api/v2/forms/{form_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetForm.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetForm.Responses.$200
      | Paths.GetForm.Responses.$400
      | Paths.GetForm.Responses.$401
      | Paths.GetForm.Responses.$404
    >
  }
  ['/api/v2/forms/{form_id}/form_data']: {
    'get'(
      parameters?: Parameters<Paths.GetFormFormData.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetFormFormData.Responses.$200
      | Paths.GetFormFormData.Responses.$400
      | Paths.GetFormFormData.Responses.$401
      | Paths.GetFormFormData.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateForm.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateForm.Responses.$201
      | Paths.CreateForm.Responses.$404
      | Paths.CreateForm.Responses.$422
    >
  }
  ['/api/v2/form_data/{form_datum_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetFormDataById.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetFormDataById.Responses.$200
      | Paths.GetFormDataById.Responses.$400
      | Paths.GetFormDataById.Responses.$401
      | Paths.GetFormDataById.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/related_entities']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationRelatedEntitie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationRelatedEntitie.Responses.$200
      | Paths.ListKycApplicationRelatedEntitie.Responses.$400
      | Paths.ListKycApplicationRelatedEntitie.Responses.$401
      | Paths.ListKycApplicationRelatedEntitie.Responses.$404
      | Paths.ListKycApplicationRelatedEntitie.Responses.$409
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateKycApplicationRelatedEntitie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$201
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$400
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$401
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$404
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$406
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$409
      | Paths.CreateKycApplicationRelatedEntitie.Responses.$422
    >
  }
  ['/api/v2/related_entities/{related_entity_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetRelatedEntitie.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetRelatedEntitie.Responses.$200
      | Paths.GetRelatedEntitie.Responses.$400
      | Paths.GetRelatedEntitie.Responses.$401
      | Paths.GetRelatedEntitie.Responses.$404
    >

    'delete'(
      parameters?: Parameters<Paths.DeleteRelatedEntitie.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteRelatedEntitie.Responses.$204
      | Paths.DeleteRelatedEntitie.Responses.$400
      | Paths.DeleteRelatedEntitie.Responses.$401
      | Paths.DeleteRelatedEntitie.Responses.$404
      | Paths.DeleteRelatedEntitie.Responses.$422
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/pdf_report_summaries']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationPdfReportSummarie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationPdfReportSummarie.Responses.$200
      | Paths.ListKycApplicationPdfReportSummarie.Responses.$400
      | Paths.ListKycApplicationPdfReportSummarie.Responses.$401
      | Paths.ListKycApplicationPdfReportSummarie.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateKycApplicationPdfReportSummarie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$201
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$400
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$401
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$404
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$406
      | Paths.CreateKycApplicationPdfReportSummarie.Responses.$429
    >
  }
  ['/api/v2/pdf_report_summaries/{pdf_report_summary_id}/']: {
    'get'(
      parameters?: Parameters<Paths.GetPdfReportSummarie.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetPdfReportSummarie.Responses.$200
      | Paths.GetPdfReportSummarie.Responses.$400
      | Paths.GetPdfReportSummarie.Responses.$401
      | Paths.GetPdfReportSummarie.Responses.$404
    >
  }
  ['/api/v2/pdf_report_summaries/{pdf_summary_report_id}/download']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetPdfReportSummarieDownload.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetPdfReportSummarieDownload.Responses.$302
      | Paths.GetPdfReportSummarieDownload.Responses.$400
      | Paths.GetPdfReportSummarieDownload.Responses.$401
      | Paths.GetPdfReportSummarieDownload.Responses.$404
      | Paths.GetPdfReportSummarieDownload.Responses.$406
      | Paths.GetPdfReportSummarieDownload.Responses.$409
    >
  }
  ['/api/v2/users']: {
    'get'(
      parameters?: Parameters<Paths.ListUser.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListUser.Responses.$200
      | Paths.ListUser.Responses.$400
      | Paths.ListUser.Responses.$401
      | Paths.ListUser.Responses.$403
    >
  }
  ['/api/v2/users/{user_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetUser.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetUser.Responses.$200
      | Paths.GetUser.Responses.$400
      | Paths.GetUser.Responses.$401
      | Paths.GetUser.Responses.$404
    >
  }
  ['/api/v2/risk_rating_evaluations/{risk_rating_evaluation_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetRiskRatingEvaluation.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetRiskRatingEvaluation.Responses.$200
      | Paths.GetRiskRatingEvaluation.Responses.$400
      | Paths.GetRiskRatingEvaluation.Responses.$401
      | Paths.GetRiskRatingEvaluation.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/risk_rating_evaluations']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationRiskRatingEvaluation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$200
      | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$400
      | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$401
      | Paths.ListKycApplicationRiskRatingEvaluation.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/risk_ratings']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationRiskRating.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationRiskRating.Responses.$200
      | Paths.ListKycApplicationRiskRating.Responses.$400
      | Paths.ListKycApplicationRiskRating.Responses.$401
      | Paths.ListKycApplicationRiskRating.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateKycApplicationRiskRating.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplicationRiskRating.Responses.$201
      | Paths.CreateKycApplicationRiskRating.Responses.$400
      | Paths.CreateKycApplicationRiskRating.Responses.$401
      | Paths.CreateKycApplicationRiskRating.Responses.$404
      | Paths.CreateKycApplicationRiskRating.Responses.$406
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/risk_ratings/{evaluation_id}']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetKycApplicationRiskRating.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetKycApplicationRiskRating.Responses.$200
      | Paths.GetKycApplicationRiskRating.Responses.$400
      | Paths.GetKycApplicationRiskRating.Responses.$401
      | Paths.GetKycApplicationRiskRating.Responses.$404
    >

    'patch'(
      parameters?:
        | Parameters<Paths.UpdateKycApplicationRiskRating.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateKycApplicationRiskRating.Responses.$200
      | Paths.UpdateKycApplicationRiskRating.Responses.$400
      | Paths.UpdateKycApplicationRiskRating.Responses.$401
      | Paths.UpdateKycApplicationRiskRating.Responses.$404
      | Paths.UpdateKycApplicationRiskRating.Responses.$406
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/risk_ratings/questions']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationRiskRatingsQuestion.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationRiskRatingsQuestion.Responses.$200
      | Paths.ListKycApplicationRiskRatingsQuestion.Responses.$404
    >
  }
  ['/api/v2/bank_account_schemas/{kyc_application_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetBankAccountSchema.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetBankAccountSchema.Responses.$200
      | Paths.GetBankAccountSchema.Responses.$400
      | Paths.GetBankAccountSchema.Responses.$401
      | Paths.GetBankAccountSchema.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/bank_accounts']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationBankAccount.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationBankAccount.Responses.$200
      | Paths.ListKycApplicationBankAccount.Responses.$400
      | Paths.ListKycApplicationBankAccount.Responses.$401
      | Paths.ListKycApplicationBankAccount.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateKycApplicationBankAccount.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplicationBankAccount.Responses.$201
      | Paths.CreateKycApplicationBankAccount.Responses.$400
      | Paths.CreateKycApplicationBankAccount.Responses.$401
      | Paths.CreateKycApplicationBankAccount.Responses.$404
      | Paths.CreateKycApplicationBankAccount.Responses.$406
      | Paths.CreateKycApplicationBankAccount.Responses.$422
    >
  }
  ['/api/v2/bank_accounts/{bank_account_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetBankAccount.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetBankAccount.Responses.$200
      | Paths.GetBankAccount.Responses.$400
      | Paths.GetBankAccount.Responses.$401
      | Paths.GetBankAccount.Responses.$404
    >
  }
  ['/api/v2/bank_account_verifications/{bank_account_verification_id}']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetBankAccountVerification.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetBankAccountVerification.Responses.$200
      | Paths.GetBankAccountVerification.Responses.$400
      | Paths.GetBankAccountVerification.Responses.$401
      | Paths.GetBankAccountVerification.Responses.$404
    >

    'patch'(
      parameters?:
        | Parameters<Paths.UpdateBankAccountVerification.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateBankAccountVerification.Responses.$200
      | Paths.UpdateBankAccountVerification.Responses.$400
      | Paths.UpdateBankAccountVerification.Responses.$401
      | Paths.UpdateBankAccountVerification.Responses.$404
      | Paths.UpdateBankAccountVerification.Responses.$406
    >
  }
  ['/api/v2/bank_accounts/{bank_account_id}/bank_account_verifications']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListBankAccountBankAccountVerification.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListBankAccountBankAccountVerification.Responses.$200
      | Paths.ListBankAccountBankAccountVerification.Responses.$400
      | Paths.ListBankAccountBankAccountVerification.Responses.$401
      | Paths.ListBankAccountBankAccountVerification.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/consumer_report_queries']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationConsumerReportQuerie.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationConsumerReportQuerie.Responses.$200
      | Paths.ListKycApplicationConsumerReportQuerie.Responses.$400
      | Paths.ListKycApplicationConsumerReportQuerie.Responses.$401
      | Paths.ListKycApplicationConsumerReportQuerie.Responses.$404
    >
  }
  ['/api/v2/consumer_report_queries/{id}']: {
    'get'(
      parameters?: Parameters<Paths.GetConsumerReportQuerie.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetConsumerReportQuerie.Responses.$200
      | Paths.GetConsumerReportQuerie.Responses.$400
      | Paths.GetConsumerReportQuerie.Responses.$401
      | Paths.GetConsumerReportQuerie.Responses.$404
    >
  }
  ['/api/v2/consumer_report_queries/{id}/download']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetConsumerReportQuerieDownload.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetConsumerReportQuerieDownload.Responses.$302
      | Paths.GetConsumerReportQuerieDownload.Responses.$400
      | Paths.GetConsumerReportQuerieDownload.Responses.$401
      | Paths.GetConsumerReportQuerieDownload.Responses.$404
      | Paths.GetConsumerReportQuerieDownload.Responses.$406
      | Paths.GetConsumerReportQuerieDownload.Responses.$409
    >
  }
  ['/api/v2/onboarding_invitations']: {
    'post'(
      parameters?:
        | Parameters<Paths.CreateOnboardingInvitation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateOnboardingInvitation.Responses.$201
      | Paths.CreateOnboardingInvitation.Responses.$400
      | Paths.CreateOnboardingInvitation.Responses.$401
      | Paths.CreateOnboardingInvitation.Responses.$403
      | Paths.CreateOnboardingInvitation.Responses.$406
    >
  }
  ['/api/v2/onboarding_invitations/{kyc_application_id}']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateOnboardingInvitation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateOnboardingInvitation.Responses.$200
      | Paths.UpdateOnboardingInvitation.Responses.$400
      | Paths.UpdateOnboardingInvitation.Responses.$401
      | Paths.UpdateOnboardingInvitation.Responses.$404
      | Paths.UpdateOnboardingInvitation.Responses.$406
      | Paths.UpdateOnboardingInvitation.Responses.$422
    >
  }
  ['/api/v2/forms/{form_id}/form_pdf_exports']: {
    'get'(
      parameters?: Parameters<Paths.ListFormFormPdfExport.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListFormFormPdfExport.Responses.$200
      | Paths.ListFormFormPdfExport.Responses.$400
      | Paths.ListFormFormPdfExport.Responses.$401
      | Paths.ListFormFormPdfExport.Responses.$404
    >

    'post'(
      parameters?: Parameters<Paths.CreateFormFormPdfExport.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateFormFormPdfExport.Responses.$201
      | Paths.CreateFormFormPdfExport.Responses.$400
      | Paths.CreateFormFormPdfExport.Responses.$401
      | Paths.CreateFormFormPdfExport.Responses.$404
      | Paths.CreateFormFormPdfExport.Responses.$406
      | Paths.CreateFormFormPdfExport.Responses.$429
    >
  }
  ['/api/v2/form_pdf_exports/{form_pdf_export_id}/']: {
    'get'(
      parameters?: Parameters<Paths.GetFormPdfExport.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetFormPdfExport.Responses.$200
      | Paths.GetFormPdfExport.Responses.$400
      | Paths.GetFormPdfExport.Responses.$401
      | Paths.GetFormPdfExport.Responses.$404
    >
  }
  ['/api/v2/form_pdf_exports/{form_pdf_export_id}/download']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetFormPdfExportDownload.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetFormPdfExportDownload.Responses.$302
      | Paths.GetFormPdfExportDownload.Responses.$400
      | Paths.GetFormPdfExportDownload.Responses.$401
      | Paths.GetFormPdfExportDownload.Responses.$404
      | Paths.GetFormPdfExportDownload.Responses.$406
      | Paths.GetFormPdfExportDownload.Responses.$409
    >
  }
  ['/api/v2/kyc_entity_templates/{kyc_entity_template_id}/designation_templates']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycEntityTemplateDesignationTemplate.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$200
      | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$400
      | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$401
      | Paths.ListKycEntityTemplateDesignationTemplate.Responses.$404
    >
  }
  ['/api/v2/related_entity_designations/{related_entity_designation_id}']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetRelatedEntityDesignation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetRelatedEntityDesignation.Responses.$200
      | Paths.GetRelatedEntityDesignation.Responses.$400
      | Paths.GetRelatedEntityDesignation.Responses.$401
      | Paths.GetRelatedEntityDesignation.Responses.$404
    >

    'delete'(
      parameters?:
        | Parameters<Paths.DeleteRelatedEntityDesignation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.DeleteRelatedEntityDesignation.Responses.$204
      | Paths.DeleteRelatedEntityDesignation.Responses.$400
      | Paths.DeleteRelatedEntityDesignation.Responses.$401
      | Paths.DeleteRelatedEntityDesignation.Responses.$404
    >
  }
  ['/api/v2/related_entities/{related_entity_id}/related_entity_designations']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListRelatedEntitieRelatedEntityDesignation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$200
      | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$400
      | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$401
      | Paths.ListRelatedEntitieRelatedEntityDesignation.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateRelatedEntitieRelatedEntityDesignation.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$201
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$400
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$401
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$404
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$406
      | Paths.CreateRelatedEntitieRelatedEntityDesignation.Responses.$422
    >
  }
  ['/api/v2/products/{product_id}/product_product_addons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListProductProductProductAddon.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProductProductProductAddon.Responses.$200
      | Paths.ListProductProductProductAddon.Responses.$400
      | Paths.ListProductProductProductAddon.Responses.$401
      | Paths.ListProductProductProductAddon.Responses.$404
    >
  }
  ['/api/v2/account_applications/{account_application_id}/account_addons']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListAccountApplicationAccountAddon.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListAccountApplicationAccountAddon.Responses.$200
      | Paths.ListAccountApplicationAccountAddon.Responses.$400
      | Paths.ListAccountApplicationAccountAddon.Responses.$401
      | Paths.ListAccountApplicationAccountAddon.Responses.$404
    >
  }
  ['/api/v2/account_addons/{account_addon_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetAccountAddon.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAccountAddon.Responses.$200
      | Paths.GetAccountAddon.Responses.$400
      | Paths.GetAccountAddon.Responses.$401
      | Paths.GetAccountAddon.Responses.$404
    >
  }
  ['/api/v2/products/{product_id}/disclosures']: {
    'get'(
      parameters?: Parameters<Paths.ListProductDisclosure.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProductDisclosure.Responses.$200
      | Paths.ListProductDisclosure.Responses.$400
      | Paths.ListProductDisclosure.Responses.$401
      | Paths.ListProductDisclosure.Responses.$403
      | Paths.ListProductDisclosure.Responses.$404
    >
  }
  ['/api/v2/disclosures']: {
    'get'(
      parameters?: Parameters<Paths.ListDisclosure.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListDisclosure.Responses.$200
      | Paths.ListDisclosure.Responses.$400
      | Paths.ListDisclosure.Responses.$401
      | Paths.ListDisclosure.Responses.$403
    >
  }
  ['/api/v2/disclosures/{disclosure_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetDisclosure.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetDisclosure.Responses.$200
      | Paths.GetDisclosure.Responses.$400
      | Paths.GetDisclosure.Responses.$401
      | Paths.GetDisclosure.Responses.$404
    >
  }
  ['/api/v2/products']: {
    'get'(
      parameters?: Parameters<Paths.ListProduct.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProduct.Responses.$200
      | Paths.ListProduct.Responses.$400
      | Paths.ListProduct.Responses.$401
      | Paths.ListProduct.Responses.$403
    >
  }
  ['/api/v2/products/{id}']: {
    'get'(
      parameters?: Parameters<Paths.GetProduct.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetProduct.Responses.$200
      | Paths.GetProduct.Responses.$400
      | Paths.GetProduct.Responses.$401
      | Paths.GetProduct.Responses.$404
    >
  }
  ['/api/v2/kyc_entity_templates/{kyc_entity_template_id}/products']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycEntityTemplateProduct.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycEntityTemplateProduct.Responses.$200
      | Paths.ListKycEntityTemplateProduct.Responses.$400
      | Paths.ListKycEntityTemplateProduct.Responses.$401
      | Paths.ListKycEntityTemplateProduct.Responses.$404
    >
  }
  ['/api/v2/account_applications/{id}/account_summary_pdf']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetAccountApplicationAccountSummaryPdf.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAccountApplicationAccountSummaryPdf.Responses.$200
      | Paths.GetAccountApplicationAccountSummaryPdf.Responses.$404
    >
  }
  ['/api/v2/kyc_entity_templates/{kyc_entity_template_id}/disclosures']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycEntityTemplateDisclosure.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycEntityTemplateDisclosure.Responses.$200
      | Paths.ListKycEntityTemplateDisclosure.Responses.$400
      | Paths.ListKycEntityTemplateDisclosure.Responses.$401
      | Paths.ListKycEntityTemplateDisclosure.Responses.$403
      | Paths.ListKycEntityTemplateDisclosure.Responses.$404
    >
  }
  ['/api/v2/account_applications/{account_application_id}/data_requirements']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListAccountApplicationDataRequirement.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListAccountApplicationDataRequirement.Responses.$200
      | Paths.ListAccountApplicationDataRequirement.Responses.$400
      | Paths.ListAccountApplicationDataRequirement.Responses.$401
      | Paths.ListAccountApplicationDataRequirement.Responses.$404
    >
  }
  ['/api/v2/product_product_addons/disclosures']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListProductProductAddonsDisclosure.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListProductProductAddonsDisclosure.Responses.$200
      | Paths.ListProductProductAddonsDisclosure.Responses.$400
      | Paths.ListProductProductAddonsDisclosure.Responses.$401
      | Paths.ListProductProductAddonsDisclosure.Responses.$403
    >
  }
  ['/api/v2/account_applications/{id}/account_summary']: {
    'get'(
      parameters?:
        | Parameters<Paths.GetAccountApplicationAccountSummary.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAccountApplicationAccountSummary.Responses.$200
      | Paths.GetAccountApplicationAccountSummary.Responses.$404
    >
  }
  ['/api/v2/kyc_applications/{kyc_application_id}/account_applications']: {
    'get'(
      parameters?:
        | Parameters<Paths.ListKycApplicationAccountApplication.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.ListKycApplicationAccountApplication.Responses.$200
      | Paths.ListKycApplicationAccountApplication.Responses.$400
      | Paths.ListKycApplicationAccountApplication.Responses.$401
      | Paths.ListKycApplicationAccountApplication.Responses.$404
    >

    'post'(
      parameters?:
        | Parameters<Paths.CreateKycApplicationAccountApplication.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.CreateKycApplicationAccountApplication.Responses.$201
      | Paths.CreateKycApplicationAccountApplication.Responses.$400
      | Paths.CreateKycApplicationAccountApplication.Responses.$401
      | Paths.CreateKycApplicationAccountApplication.Responses.$404
      | Paths.CreateKycApplicationAccountApplication.Responses.$406
    >
  }
  ['/api/v2/account_applications/{account_application_id}']: {
    'get'(
      parameters?: Parameters<Paths.GetAccountApplication.OperationParameters> | null | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.GetAccountApplication.Responses.$200
      | Paths.GetAccountApplication.Responses.$400
      | Paths.GetAccountApplication.Responses.$401
      | Paths.GetAccountApplication.Responses.$404
    >

    'patch'(
      parameters?:
        | Parameters<Paths.UpdateAccountApplication.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateAccountApplication.Responses.$200
      | Paths.UpdateAccountApplication.Responses.$400
      | Paths.UpdateAccountApplication.Responses.$401
      | Paths.UpdateAccountApplication.Responses.$404
      | Paths.UpdateAccountApplication.Responses.$406
    >
  }
  ['/api/v2/account_applications/{id}/disclosure_acceptances']: {
    'patch'(
      parameters?:
        | Parameters<Paths.UpdateAccountApplicationDisclosureAcceptances.OperationParameters>
        | null
        | undefined,
      payload?: any,
      config?: AxiosRequestConfig
    ): OperationResponse<
      | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$200
      | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$400
      | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$401
      | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$404
      | Paths.UpdateAccountApplicationDisclosureAcceptances.Responses.$406
    >
  }
}

export type Client = OpenAPIClient<OperationMethods, PathsDictionary>
